from ..value_set import ValueSet


class AntiHepatitisAIggAntigenTest(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for the detection of anti Hepatitis A immunoglobulin (IgG) antibodies.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for the IgG antibody to Hepatitis A in serum.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS117v10
    """

    VALUE_SET_NAME = 'Anti Hepatitis A IgG Antigen Test'
    OID = '2.16.840.1.113883.3.464.1003.198.12.1033'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '32018-4',  # Hepatitis A virus IgG Ab [Presence] in Serum
        '40724-7',  # Hepatitis A virus IgG Ab [Presence] in Serum by Immunoassay
        '51913-2',  # Hepatitis A virus IgG+IgM Ab [Presence] in Serum
    }


class AntiHepatitisBVirusSurfaceAb(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for the detection of the anti Hepatitis B surface antigen.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for the anti-surface antibody to Hepatitis B virus in serum.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS117v10
    """

    VALUE_SET_NAME = 'Anti Hepatitis B Virus Surface Ab'
    OID = '2.16.840.1.113883.3.464.1003.198.12.1073'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '10900-9',  # Hepatitis B virus surface Ab [Presence] in Serum by Immunoassay
        '22322-2',  # Hepatitis B virus surface Ab [Presence] in Serum
        '39535-0',  # Hepatitis B virus surface Ab [Presence] in Serum by Radioimmunoassay (RIA)
        '48070-7',  # Hepatitis B virus surface IgG Ab [Presence] in Serum by Immunoassay
        '49177-9',  # Hepatitis B virus surface IgG Ab [Presence] in Serum
        '75409-3',  # Hepatitis B virus surface Ab [Presence] in Serum, Plasma or Blood by Rapid immunoassay
    }


class MeaslesAntibodyTestIggAntibodyTiter(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for laboratory tests for the titer of measles IgG antibody.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for the measles IgG titer in serum or cerebrospinal fluid.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS117v10
    """

    VALUE_SET_NAME = 'Measles Antibody Test (IgG Antibody Titer)'
    OID = '2.16.840.1.113883.3.464.1003.198.12.1059'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '21500-4',  # Measles virus IgG Ab [Titer] in Cerebral spinal fluid by Immunofluorescence
        '21501-2',  # Measles virus IgG Ab [Titer] in Serum by Immunofluorescence
        '22501-1',  # Measles virus IgG Ab [Titer] in Cerebral spinal fluid
        '22502-9',  # Measles virus IgG Ab [Titer] in Serum
    }


class MeaslesAntibodyTestIggAntibodyPresence(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for laboratory tests for the presence of measles IgG antibody.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for the presence of measles IgG in serum, cerebrospinal fluid or body fluid.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS117v10
    """

    VALUE_SET_NAME = 'Measles Antibody Test (IgG Antibody presence)'
    OID = '2.16.840.1.113883.3.464.1003.198.12.1060'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '20479-2',  # Measles virus IgG Ab [Presence] in Serum
        '35275-7',  # Measles virus IgG Ab [Presence] in Serum by Immunoassay
        '40648-8',  # Measles virus IgG Ab [Presence] in Cerebral spinal fluid by Immunoassay
        '41132-2',  # Measles virus IgG Ab [Presence] in Cerebral spinal fluid
        '53536-9',  # Measles virus IgG Ab [Presence] in Body fluid by Immunoassay
    }


class MumpsAntibodyTestIggAntibodyTiter(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for laboratory tests for the titer of mumps IgG antibody.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for the mumps IgG titer in serum or cerebrospinal fluid.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS117v10
    """

    VALUE_SET_NAME = 'Mumps Antibody Test (IgG Antibody Titer)'
    OID = '2.16.840.1.113883.3.464.1003.198.12.1061'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '21401-5',  # Mumps virus IgG Ab [Titer] in Cerebral spinal fluid by Immunofluorescence
        '22416-2',  # Mumps virus IgG Ab [Titer] in Cerebral spinal fluid
        '22417-0',  # Mumps virus IgG Ab [Titer] in Serum
        '6477-4',  # Mumps virus IgG Ab [Titer] in Serum by Immunofluorescence
    }


class MumpsAntibodyTestIggAntibodyPresence(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for laboratory tests for the presence of mumps IgG antibody.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for the presence of mumps IgG in serum, cerebrospinal fluid or body fluid.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS117v10
    """

    VALUE_SET_NAME = 'Mumps Antibody Test (IgG Antibody presence)'
    OID = '2.16.840.1.113883.3.464.1003.198.12.1062'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '22415-4',  # Mumps virus IgG Ab [Presence] in Serum
        '39011-2',  # Mumps virus IgG Ab [Presence] in Cerebral spinal fluid
        '40737-9',  # Mumps virus IgG Ab [Presence] in Cerebral spinal fluid by Immunoassay
        '6476-6',  # Mumps virus IgG Ab [Presence] in Serum by Immunoassay
        '74422-7',  # Mumps virus IgG Ab [Presence] in Body fluid by Immunoassay
    }


class RubellaAntibodyTestIggAntibodyTiter(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for laboratory tests for the titer of rubella IgG antibody.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for the rubella IgG titer in serum or cerebrospinal fluid.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS117v10
    """

    VALUE_SET_NAME = 'Rubella Antibody Test (IgG Antibody Titer)'
    OID = '2.16.840.1.113883.3.464.1003.198.12.1063'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '41763-4',  # Rubella virus IgG Ab [Titer] in Serum
        '46110-3',  # Rubella virus IgG Ab [Titer] in Cerebral spinal fluid
    }


class RubellaAntibodyTestIggAntibodyPresence(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for laboratory tests for the presence of rubella IgG antibody.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for the presence of rubella IgG in serum, cerebrospinal fluid, body fluid.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS117v10
    """

    VALUE_SET_NAME = 'Rubella Antibody Test (IgG Antibody presence)'
    OID = '2.16.840.1.113883.3.464.1003.198.12.1064'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '25514-1',  # Rubella virus IgG Ab [Presence] in Serum
        '39013-8',  # Rubella virus IgG Ab [Presence] in Cerebral spinal fluid
        '40667-8',  # Rubella virus IgG Ab [Presence] in Serum or Plasma by Immunoassay
        '40668-6',  # Rubella virus IgG Ab [Presence] in Cerebral spinal fluid by Immunoassay
        '63462-6',  # Rubella virus IgG Ab [Presence] in Serum by Latex agglutination
        '74415-1',  # Rubella virus IgG Ab [Presence] in Body fluid by Immunoassay
    }


class VaricellaZosterAntibodyTestIggAntibodyPresence(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for laboratory tests for the presence of varicella zoster IgG antibody.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for the presence of varicella zoster IgG in serum, cerebrospinal fluid or body fluid.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS117v10
    """

    VALUE_SET_NAME = 'Varicella Zoster Antibody Test (IgG Antibody Presence)'
    OID = '2.16.840.1.113883.3.464.1003.198.12.1067'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '15410-4',  # Varicella zoster virus IgG Ab [Presence] in Serum by Immunoassay
        '19162-7',  # Varicella zoster virus IgG Ab [Presence] in Serum
        '41512-5',  # Varicella zoster virus IgG Ab [Presence] in Serum by Immunofluorescence
        '42537-1',  # Varicella zoster virus IgG Ab [Presence] in Cerebral spinal fluid
        '53534-4',  # Varicella zoster virus IgG Ab [Presence] in Body fluid by Immunoassay
    }


class VaricellaZosterAntibodyTestIggAntibodyTiter(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for laboratory tests for the titer of varicella zoster IgG antibody.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for the varicella zoster IgG titer in serum or cerebrospinal fluid.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS117v10
    """

    VALUE_SET_NAME = 'Varicella Zoster Antibody Test (IgG Antibody Titer)'
    OID = '2.16.840.1.113883.3.464.1003.198.12.1066'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '21595-4',  # Varicella zoster virus IgG Ab [Titer] in Cerebral spinal fluid by Immunofluorescence
        '22601-9',  # Varicella zoster virus IgG Ab [Titer] in Cerebral spinal fluid
        '22602-7',  # Varicella zoster virus IgG Ab [Titer] in Serum
        '6569-8',  # Varicella zoster virus IgG Ab [Titer] in Serum by Immunofluorescence
    }


class Hba1CLaboratoryTest(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for hemoglobin A1c tests.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for hemoglobin A1c.

    **Exclusion Criteria:** Excludes concepts that represent hemoglobin A1c laboratory tests that use the International Federation of Clinical Chemistry and Laboratory Medicine (IFCC) protocol and Japanese Diabetes Society (JDS)/Japanese Society of Clinical Chemistry (JSCC) protocol and exclude concepts that represent an order only.

    ** Used in:** CMS122v10
    """

    VALUE_SET_NAME = 'HbA1c Laboratory Test'
    OID = '2.16.840.1.113883.3.464.1003.198.12.1013'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '17856-6',  # Hemoglobin A1c/Hemoglobin.total in Blood by HPLC
        '4548-4',  # Hemoglobin A1c/Hemoglobin.total in Blood
        '4549-2',  # Hemoglobin A1c/Hemoglobin.total in Blood by Electrophoresis
    }


class HpvTest(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for human papilloma viruses (HPV).

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for high-risk HPV in cervical samples.

    **Exclusion Criteria:** Excludes concepts that represent tests for high-risk HPV conducted on non-cervical samples and tests that are ordered only.

    ** Used in:** CMS124v10
    """

    VALUE_SET_NAME = 'HPV Test'
    OID = '2.16.840.1.113883.3.464.1003.110.12.1059'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '21440-3',  # Human papilloma virus 16+18+31+33+35+45+51+52+56 DNA [Presence] in Cervix by Probe
        '30167-1',  # Human papilloma virus 16+18+31+33+35+39+45+51+52+56+58+59+68 DNA [Presence] in Cervix by Probe with signal amplification
        '38372-9',  # Human papilloma virus 6+11+16+18+31+33+35+39+42+43+44+45+51+52+56+58+59+68 DNA [Presence] in Cervix by Probe with signal amplification
        '59263-4',  # Human papilloma virus 16 DNA [Presence] in Cervix by Probe with signal amplification
        '59264-2',  # Human papilloma virus 18 DNA [Presence] in Cervix by Probe with signal amplification
        '59420-0',  # Human papilloma virus 16+18+31+33+35+39+45+51+52+56+58+59+66+68 DNA [Presence] in Cervix by Probe with signal amplification
        '69002-4',  # Human papilloma virus E6+E7 mRNA [Presence] in Cervix by NAA with probe detection
        '71431-1',  # Human papilloma virus 31+33+35+39+45+51+52+56+58+59+66+68 DNA [Presence] in Cervix by NAA with probe detection
        '75694-0',  # Human papilloma virus 18+45 E6+E7 mRNA [Presence] in Cervix by NAA with probe detection
        '77379-6',  # Human papilloma virus 16 and 18 and 31+33+35+39+45+51+52+56+58+59+66+68 DNA [Interpretation] in Cervix
        '77399-4',  # Human papilloma virus 16 DNA [Presence] in Cervix by NAA with probe detection
        '77400-0',  # Human papilloma virus 18 DNA [Presence] in Cervix by NAA with probe detection
        '82354-2',  # Human papilloma virus 16 and 18+45 E6+E7 mRNA [Identifier] in Cervix by NAA with probe detection
        '82456-5',  # Human papilloma virus 16 E6+E7 mRNA [Presence] in Cervix by NAA with probe detection
        '82675-0',  # Human papilloma virus 16+18+31+33+35+39+45+51+52+56+58+59+66+68 DNA [Presence] in Cervix by NAA with probe detection
    }


class PapTest(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for cervical cytology tests.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for cervical cytology.

    **Exclusion Criteria:** Excludes concepts that represent tests for cervical cytology conducted on non-cervical or non-vaginal samples and tests that are ordered only.

    ** Used in:** CMS124v10, CMS153v10
    """

    VALUE_SET_NAME = 'Pap Test'
    OID = '2.16.840.1.113883.3.464.1003.108.12.1017'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '10524-7',  # Microscopic observation [Identifier] in Cervix by Cyto stain
        '18500-9',  # Microscopic observation [Identifier] in Cervix by Cyto stain.thin prep
        '19762-4',  # General categories [Interpretation] of Cervical or vaginal smear or scraping by Cyto stain
        '19764-0',  # Statement of adequacy [Interpretation] of Cervical or vaginal smear or scraping by Cyto stain
        '19765-7',  # Microscopic observation [Identifier] in Cervical or vaginal smear or scraping by Cyto stain
        '19766-5',  # Microscopic observation [Identifier] in Cervical or vaginal smear or scraping by Cyto stain Narrative
        '19774-9',  # Cytology study comment Cervical or vaginal smear or scraping Cyto stain
        '33717-0',  # Cytology Cervical or vaginal smear or scraping study
        '47527-7',  # Cytology report of Cervical or vaginal smear or scraping Cyto stain.thin prep
        '47528-5',  # Cytology report of Cervical or vaginal smear or scraping Cyto stain
    }


class ProstateSpecificAntigenTest(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for laboratory tests measuring the prostate specific antigen (PSA).

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent a laboratory test for a prostate specific antigen (PSA).

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS129v11
    """

    VALUE_SET_NAME = 'Prostate Specific Antigen Test'
    OID = '2.16.840.1.113883.3.526.3.401'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '10508-0',  # Prostate specific Ag [Presence] in Tissue by Immune stain
        '10886-0',  # Prostate Specific Ag Free [Mass/volume] in Serum or Plasma
        '12841-3',  # Prostate Specific Ag Free/Prostate specific Ag.total in Serum or Plasma
        '15323-9',  # Prostate specific Ag/Prostate volume calculated from height, width and length
        '15324-7',  # Prostate specific Ag/Prostate volume calculated from planimetry
        '15325-4',  # Prostate specific Ag/Prostate volume calculated
        '19195-7',  # Prostate specific Ag [Units/volume] in Serum or Plasma
        '19201-3',  # Prostate Specific Ag Free [Units/volume] in Serum or Plasma
        '2857-1',  # Prostate specific Ag [Mass/volume] in Serum or Plasma
        '33667-7',  # Prostate specific Ag.protein bound [Mass/volume] in Serum or Plasma
        '34611-4',  # Prostate specific Ag [Mass/volume] in Urine
        '35741-8',  # Prostate specific Ag [Mass/volume] in Serum or Plasma by Detection limit <= 0.01 ng/mL
    }


class FitDna(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for fecal immunochemical (FIT) DNA tests.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent fecal immunochemical (FIT) DNA tests.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS130v10
    """

    VALUE_SET_NAME = 'FIT DNA'
    OID = '2.16.840.1.113883.3.464.1003.108.12.1039'
    DEFINITION_VERSION = '20171219'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '77353-1',  # Noninvasive colorectal cancer DNA and occult blood screening [Interpretation] in Stool Narrative
        '77354-9',  # Noninvasive colorectal cancer DNA and occult blood screening [Presence] in Stool
    }


class FecalOccultBloodTestFobt(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for fecal occult blood tests.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for occult blood in stool.

    **Exclusion Criteria:** Excludes concepts that represent an order only and test for occult blood in other body fluids.

    ** Used in:** CMS130v10
    """

    VALUE_SET_NAME = 'Fecal Occult Blood Test (FOBT)'
    OID = '2.16.840.1.113883.3.464.1003.198.12.1011'
    DEFINITION_VERSION = '20171219'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '12503-9',  # Hemoglobin.gastrointestinal [Presence] in Stool --4th specimen
        '12504-7',  # Hemoglobin.gastrointestinal [Presence] in Stool --5th specimen
        '14563-1',  # Hemoglobin.gastrointestinal [Presence] in Stool --1st specimen
        '14564-9',  # Hemoglobin.gastrointestinal [Presence] in Stool --2nd specimen
        '14565-6',  # Hemoglobin.gastrointestinal [Presence] in Stool --3rd specimen
        '2335-8',  # Hemoglobin.gastrointestinal [Presence] in Stool
        '27396-1',  # Hemoglobin.gastrointestinal [Mass/mass] in Stool
        '27401-9',  # Hemoglobin.gastrointestinal [Presence] in Stool --6th specimen
        '27925-7',  # Hemoglobin.gastrointestinal [Presence] in Stool --7th specimen
        '27926-5',  # Hemoglobin.gastrointestinal [Presence] in Stool --8th specimen
        '29771-3',  # Hemoglobin.gastrointestinal.lower [Presence] in Stool by Immunoassay
        '56490-6',  # Hemoglobin.gastrointestinal.lower [Presence] in Stool by Immunoassay --2nd specimen
        '56491-4',  # Hemoglobin.gastrointestinal.lower [Presence] in Stool by Immunoassay --3rd specimen
        '57905-2',  # Hemoglobin.gastrointestinal.lower [Presence] in Stool by Immunoassay --1st specimen
        '58453-2',  # Hemoglobin.gastrointestinal.lower [Mass/volume] in Stool by Immunoassay
        '80372-6',  # Hemoglobin.gastrointestinal [Presence] in Stool by Rapid immunoassay
    }


class UrineProteinTests(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for tests that measure urine protein.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for protein in urine.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS134v10
    """

    VALUE_SET_NAME = 'Urine Protein Tests'
    OID = '2.16.840.1.113883.3.464.1003.109.12.1024'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '11218-5',  # Microalbumin [Mass/volume] in Urine by Test strip
        '12842-1',  # Protein [Mass/volume] in 12 hour Urine
        '13705-9',  # Albumin/Creatinine [Mass Ratio] in 24 hour Urine
        '13801-6',  # Protein/Creatinine [Mass Ratio] in 24 hour Urine
        '13986-5',  # Albumin/Protein.total in 24 hour Urine by Electrophoresis
        '13992-3',  # Albumin/Protein.total in Urine by Electrophoresis
        '14956-7',  # Microalbumin [Mass/time] in 24 hour Urine
        '14957-5',  # Microalbumin [Mass/volume] in Urine
        '14958-3',  # Microalbumin/Creatinine [Mass Ratio] in 24 hour Urine
        '14959-1',  # Microalbumin/Creatinine [Mass Ratio] in Urine
        '1753-3',  # Albumin [Presence] in Urine
        '1754-1',  # Albumin [Mass/volume] in Urine
        '1755-8',  # Albumin [Mass/time] in 24 hour Urine
        '1757-4',  # Albumin renal clearance in 24 hour Urine and Serum or Plasma
        '17819-4',  # Albumin/Protein.total by Electrophoresis in Urine collected for unspecified duration
        '18373-1',  # Protein [Mass/time] in 6 hour Urine
        '20454-5',  # Protein [Presence] in Urine by Test strip
        '20621-9',  # Albumin/Creatinine [Presence] in Urine by Test strip
        '21059-1',  # Albumin [Mass/volume] in 24 hour Urine
        '21482-5',  # Protein [Mass/volume] in 24 hour Urine
        '26801-1',  # Protein [Mass/time] in 12 hour Urine
        '27298-9',  # Protein [Units/volume] in Urine
        '2887-8',  # Protein [Presence] in Urine
        '2888-6',  # Protein [Mass/volume] in Urine
        '2889-4',  # Protein [Mass/time] in 24 hour Urine
        '2890-2',  # Protein/Creatinine [Mass Ratio] in Urine
        '29946-1',  # Albumin [Presence] in 24 hour Urine by Electrophoresis
        '30000-4',  # Microalbumin/Creatinine [Ratio] in Urine
        '30001-2',  # Microalbumin/Creatinine [Ratio] in Urine by Test strip
        '30003-8',  # Microalbumin [Mass/volume] in 24 hour Urine
        '32209-9',  # Protein [Presence] in 24 hour Urine by Test strip
        '32294-1',  # Albumin/Creatinine [Ratio] in Urine
        '32551-4',  # Protein [Mass] in Urine collected for unspecified duration
        '34366-5',  # Protein/Creatinine [Ratio] in Urine
        '35663-4',  # Protein [Mass/volume] in Urine collected for unspecified duration
        '40486-3',  # Protein/Creatinine [Ratio] in 24 hour Urine
        '40662-9',  # Protein [Mass/time] in 12 hour Urine --resting
        '40663-7',  # Protein [Mass/time] in 12 hour Urine --upright
        '43605-5',  # Microalbumin [Mass/volume] in 4 hour Urine
        '43606-3',  # Microalbumin [Mass/time] in 4 hour Urine
        '43607-1',  # Microalbumin [Mass/time] in 12 hour Urine
        '44292-1',  # Microalbumin/Creatinine [Mass Ratio] in 12 hour Urine
        '47558-2',  # Microalbumin/Protein.total in 24 hour Urine
        '49002-9',  # Albumin [Mass/time] in Urine collected for unspecified duration
        '49023-5',  # Microalbumin [Mass/time] in Urine collected for unspecified duration
        '50209-6',  # Albumin [Mass/time] in Urine collected for unspecified duration --supine
        '50561-0',  # Protein [Mass/volume] in Urine by Automated test strip
        '50949-7',  # Albumin [Presence] in Urine by Test strip
        '51190-7',  # Albumin [Mass/volume] in 24 hour Urine by Electrophoresis
        '53121-0',  # Protein [Mass/time] in 1 hour Urine
        '53525-2',  # Protein [Presence] in Urine by SSA method
        '53530-2',  # Microalbumin [Mass/volume] in 24 hour Urine by Detection limit <= 1.0 mg/L
        '53531-0',  # Microalbumin [Mass/volume] in Urine by Detection limit <= 1.0 mg/L
        '53532-8',  # Microalbumin [Mass/time] in 24 hour Urine by Detection limit <= 1.0 mg/L
        '56553-1',  # Microalbumin [Mass/time] in 8 hour Urine
        '57369-1',  # Microalbumin [Mass/volume] in 12 hour Urine
        '57735-3',  # Protein [Presence] in Urine by Automated test strip
        '5804-0',  # Protein [Mass/volume] in Urine by Test strip
        '58448-2',  # Microalbumin ug/min [Mass/time] in 24 hour Urine
        '58992-9',  # Protein [Mass/time] in 18 hour Urine
        '59159-4',  # Microalbumin/Creatinine [Ratio] in 24 hour Urine
        '60678-0',  # Protein/Creatinine [Mass Ratio] in 12 hour Urine
        '63474-1',  # Microalbumin [Mass/time] in 18 hour Urine
        '6941-9',  # Albumin [Mass/time] in 24 hour Urine by Electrophoresis
        '6942-7',  # Albumin [Mass/volume] in Urine by Electrophoresis
        '76401-9',  # Albumin/Creatinine [Ratio] in 24 hour Urine
        '77253-3',  # Microalbumin/Creatinine [Ratio] in Urine by Detection limit <= 1.0 mg/L
        '77254-1',  # Microalbumin/Creatinine [Ratio] in 24 hour Urine by Detection limit <= 1.0 mg/L
        '77940-5',  # Albumin [Mass/volume] by Electrophoresis in Urine collected for unspecified duration
        '89998-9',  # Albumin/Creatinine [Ratio] in Urine by Detection limit <= 3.0 mg/L
        '89999-7',  # Albumin [Mass/volume] in Urine by Detection limit <= 3.0 mg/L
        '90000-1',  # Albumin [Mass/time] in 24 hour Urine by Detection limit <= 3.0 mg/L
        '9318-7',  # Albumin/Creatinine [Mass Ratio] in Urine
        '93746-6',  # Protein catabolic rate based on 24 hour Urine [Calculated]
        '95232-5',  # Microalbumin [Presence] in Urine by Test strip
        '95233-3',  # Microalbumin/Creatinine [Presence] in Urine by Test strip
    }


class GroupAStreptococcusTest(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for group A streptococcus tests.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests to identify the presence of group A streptococcus.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS146v10
    """

    VALUE_SET_NAME = 'Group A Streptococcus Test'
    OID = '2.16.840.1.113883.3.464.1003.198.12.1012'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '11268-0',  # Streptococcus pyogenes [Presence] in Throat by Organism specific culture
        '17656-0',  # Streptococcus pyogenes [Presence] in Specimen by Organism specific culture
        '17898-8',  # Bacteria identified in Throat by Aerobe culture
        '18481-2',  # Streptococcus pyogenes Ag [Presence] in Throat
        '31971-5',  # Streptococcus pyogenes Ag [Presence] in Specimen
        '49610-9',  # Streptococcus pyogenes DNA [Identifier] in Specimen by NAA with probe detection
        '5036-9',  # Streptococcus pyogenes rRNA [Presence] in Specimen by Probe
        '60489-2',  # Streptococcus pyogenes DNA [Presence] in Throat by NAA with probe detection
        '626-2',  # Bacteria identified in Throat by Culture
        '6557-3',  # Streptococcus pyogenes Ag [Presence] in Throat by Immunofluorescence
        '6558-1',  # Streptococcus pyogenes Ag [Presence] in Specimen by Immunoassay
        '6559-9',  # Streptococcus pyogenes Ag [Presence] in Specimen by Immunofluorescence
        '68954-7',  # Streptococcus pyogenes rRNA [Presence] in Throat by Probe
        '78012-2',  # Streptococcus pyogenes Ag [Presence] in Throat by Rapid immunoassay
        '96331-4',  # Streptococcus pyogenes DNA [NCncRange] in Lower respiratory specimen Qualitative by NAA with non-probe detection
    }


class ChlamydiaScreening(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for chlamydia tests.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent screening for chlamydia infections.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS153v10
    """

    VALUE_SET_NAME = 'Chlamydia Screening'
    OID = '2.16.840.1.113883.3.464.1003.110.12.1052'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '14463-4',  # Chlamydia trachomatis [Presence] in Cervix by Organism specific culture
        '14464-2',  # Chlamydia trachomatis [Presence] in Vaginal fluid by Organism specific culture
        '14467-5',  # Chlamydia trachomatis [Presence] in Urine sediment by Organism specific culture
        '14474-1',  # Chlamydia trachomatis Ag [Presence] in Urine sediment by Immunoassay
        '14513-6',  # Chlamydia trachomatis Ag [Presence] in Urine sediment by Immunofluorescence
        '16600-9',  # Chlamydia trachomatis rRNA [Presence] in Genital specimen by Probe
        '21190-4',  # Chlamydia trachomatis DNA [Presence] in Cervix by NAA with probe detection
        '21191-2',  # Chlamydia trachomatis DNA [Presence] in Urethra by NAA with probe detection
        '21613-5',  # Chlamydia trachomatis DNA [Presence] in Specimen by NAA with probe detection
        '23838-6',  # Chlamydia trachomatis rRNA [Presence] in Genital fluid by Probe
        '31775-0',  # Chlamydia trachomatis Ag [Presence] in Urine sediment
        '31777-6',  # Chlamydia trachomatis Ag [Presence] in Specimen
        '36902-5',  # Chlamydia trachomatis+Neisseria gonorrhoeae DNA [Presence] in Specimen by NAA with probe detection
        '36903-3',  # Chlamydia trachomatis and Neisseria gonorrhoeae DNA [Identifier] in Specimen by NAA with probe detection
        '42931-6',  # Chlamydia trachomatis rRNA [Presence] in Urine by NAA with probe detection
        '43304-5',  # Chlamydia trachomatis rRNA [Presence] in Specimen by NAA with probe detection
        '43404-3',  # Chlamydia trachomatis DNA [Presence] in Specimen by Probe with signal amplification
        '43405-0',  # Chlamydia trachomatis and Neisseria gonorrhoeae DNA [Identifier] in Specimen by Probe with signal amplification
        '43406-8',  # Chlamydia trachomatis+Neisseria gonorrhoeae DNA [Presence] in Specimen by Probe with signal amplification
        '44806-8',  # Chlamydia trachomatis+Neisseria gonorrhoeae DNA [Presence] in Urine by NAA with probe detection
        '44807-6',  # Chlamydia trachomatis+Neisseria gonorrhoeae DNA [Presence] in Genital specimen by NAA with probe detection
        '45068-4',  # Chlamydia trachomatis+Neisseria gonorrhoeae DNA [Presence] in Cervix by NAA with probe detection
        '45069-2',  # Chlamydia trachomatis+Neisseria gonorrhoeae rRNA [Presence] in Genital specimen by Probe
        '45075-9',  # Chlamydia trachomatis+Neisseria gonorrhoeae rRNA [Presence] in Urethra by Probe
        '45076-7',  # Chlamydia trachomatis+Neisseria gonorrhoeae rRNA [Presence] in Specimen by Probe
        '45084-1',  # Chlamydia trachomatis DNA [Presence] in Vaginal fluid by NAA with probe detection
        '45091-6',  # Chlamydia trachomatis Ag [Presence] in Genital specimen
        '45095-7',  # Chlamydia trachomatis [Presence] in Genital specimen by Organism specific culture
        '45098-1',  # Chlamydia sp identified in Cervix by Organism specific culture
        '45100-5',  # Chlamydia sp identified in Vaginal fluid by Organism specific culture
        '47211-8',  # Chlamydia trachomatis L2 DNA [Presence] in Specimen by NAA with probe detection
        '47212-6',  # Chlamydia trachomatis DNA [Identifier] in Specimen by NAA with probe detection
        '49096-1',  # Chlamydia trachomatis DNA [Units/volume] in Specimen by NAA with probe detection
        '4993-2',  # Chlamydia trachomatis rRNA [Presence] in Specimen by Probe
        '50387-0',  # Chlamydia trachomatis rRNA [Presence] in Cervix by NAA with probe detection
        '53925-4',  # Chlamydia trachomatis rRNA [Presence] in Urethra by NAA with probe detection
        '53926-2',  # Chlamydia trachomatis rRNA [Presence] in Vaginal fluid by NAA with probe detection
        '557-9',  # Chlamydia sp identified in Genital specimen by Organism specific culture
        '560-3',  # Chlamydia sp identified in Specimen by Organism specific culture
        '6349-5',  # Chlamydia trachomatis [Presence] in Specimen by Organism specific culture
        '6354-5',  # Chlamydia trachomatis Ag [Presence] in Specimen by Immunoassay
        '6355-2',  # Chlamydia trachomatis Ag [Presence] in Specimen by Immunofluorescence
        '6356-0',  # Chlamydia trachomatis DNA [Presence] in Genital specimen by NAA with probe detection
        '6357-8',  # Chlamydia trachomatis DNA [Presence] in Urine by NAA with probe detection
        '80360-1',  # Chlamydia trachomatis+Neisseria gonorrhoeae rRNA [Presence] in Urine by NAA with probe detection
        '80361-9',  # Chlamydia trachomatis+Neisseria gonorrhoeae rRNA [Presence] in Cervix by NAA with probe detection
        '80362-7',  # Chlamydia trachomatis+Neisseria gonorrhoeae rRNA [Presence] in Vaginal fluid by NAA with probe detection
        '91860-7',  # Chlamydia trachomatis Ag [Presence] in Genital specimen by Immunofluorescence
    }


class LabTestsDuringPregnancy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for laboratory tests conducted during pregnancy.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests conducted during pregnancy.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS153v10
    """

    VALUE_SET_NAME = 'Lab Tests During Pregnancy'
    OID = '2.16.840.1.113883.3.464.1003.111.12.1007'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '15019-3',  # Alpha-1-Fetoprotein [Moles/volume] in Amniotic fluid
        '1832-5',  # Alpha-1-Fetoprotein [Mass/volume] in Amniotic fluid
        '1834-1',  # Alpha-1-Fetoprotein [Mass/volume] in Serum or Plasma
        '19171-8',  # Alpha-1-Fetoprotein [Units/volume] in Amniotic fluid
        '19176-7',  # Alpha-1-Fetoprotein [Units/volume] in Serum or Plasma
        '19177-5',  # Alpha-1-Fetoprotein [Moles/volume] in Serum or Plasma
        '20403-2',  # Fibronectin.fetal [Mass/volume] in Vaginal fluid
        '20404-0',  # Fibronectin.fetal [Presence] in Vaginal fluid
        '31993-9',  # Alpha-1-Fetoprotein [Presence] in Serum or Plasma
        '34493-7',  # PRF1 gene targeted mutation analysis in Amniotic fluid by Molecular genetics method
        '35457-1',  # Maternal cell contamination [Identifier] in Amniotic fluid Nominal
        '41067-0',  # PRF1 gene mutations found [Identifier] in Amniotic fluid by Molecular genetics method Nominal
        '41096-9',  # KEL gene allele [Genotype] in Amniotic fluid by Molecular genetics method Nominal
        '41273-4',  # Alpha-1-Fetoprotein interpretation in Amniotic fluid
        '41274-2',  # Alpha-1-Fetoprotein interpretation in Serum or Plasma
        '43798-8',  # Alpha-1-Fetoprotein [Presence] in Amniotic fluid
        '46989-0',  # CFTR gene targeted mutation analysis in Amniotic fluid by Molecular genetics method
        '48039-2',  # Fibronectin.fetal [Presence] in Specimen
        '48781-9',  # CYP21A2 gene mutations found [Identifier] in Amniotic fluid by Molecular genetics method Nominal
        '48802-3',  # Alpha-1-Fetoprotein panel - Serum or Plasma
        '49246-2',  # Alpha-1-Fetoprotein interpretation in Serum or Plasma Narrative
        '49318-9',  # Alpha-1-Fetoprotein interpretation in Amniotic fluid Narrative
        '83072-9',  # Alpha-1-Fetoprotein [Units/volume] in Serum or Plasma by Immunoassay
        '83073-7',  # Alpha-1-Fetoprotein [Mass/volume] in Serum or Plasma by Immunoassay
        '83075-2',  # Alpha-1-Fetoprotein [Mass/volume] in Amniotic fluid by Immunoassay
    }


class LabTestsForSexuallyTransmittedInfections(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for laboratory tests for sexually transmitted infections.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for sexually transmitted infections.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS153v10
    """

    VALUE_SET_NAME = 'Lab Tests for Sexually Transmitted Infections'
    OID = '2.16.840.1.113883.3.464.1003.110.12.1051'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '10705-2',  # Human papilloma virus Ag [Presence] in Tissue by Immune stain
        '11083-3',  # Human papilloma virus identified in Cervix
        '11084-1',  # Reagin Ab [Titer] in Serum
        '11481-9',  # Human papilloma virus identified in Specimen
        '11597-2',  # Treponema pallidum Ab [Units/volume] in Serum
        '12222-6',  # Human papilloma virus Ag [Presence] in Genital specimen
        '12223-4',  # Human papilloma virus 16+18 Ag [Presence] in Genital specimen
        '13288-6',  # Treponema pallidum Ab [Units/volume] in Blood by Immunofluorescence
        '13321-5',  # Human papilloma virus IgG Ab [Presence] in Serum
        '13322-3',  # Human papilloma virus IgM Ab [Presence] in Serum
        '14499-8',  # Human papilloma virus Ag [Presence] in Cervix
        '14500-3',  # Human papilloma virus Ag [Presence] in Vaginal fluid
        '14502-9',  # Human papilloma virus Ag [Presence] in Urethra
        '14503-7',  # Human papilloma virus 16+18 Ag [Presence] in Cervix
        '14504-5',  # Human papilloma virus 16+18 Ag [Presence] in Vaginal fluid
        '14506-0',  # Human papilloma virus 16+18 Ag [Presence] in Urethra
        '16280-0',  # Human papilloma virus DNA [Presence] in Specimen by Probe with amplification
        '17398-9',  # Human papilloma virus 11 Ag [Presence] in Specimen
        '17399-7',  # Human papilloma virus 16 Ag [Presence] in Specimen
        '17400-3',  # Human papilloma virus 16+18 Ag [Presence] in Specimen
        '17401-1',  # Human papilloma virus 18 Ag [Presence] in Specimen
        '17402-9',  # Human papilloma virus 31 Ag [Presence] in Specimen
        '17403-7',  # Human papilloma virus 31+33+35 Ag [Presence] in Specimen
        '17404-5',  # Human papilloma virus 33 Ag [Presence] in Specimen
        '17405-2',  # Human papilloma virus 42 Ag [Presence] in Specimen
        '17406-0',  # Human papilloma virus 43 Ag [Presence] in Specimen
        '17407-8',  # Human papilloma virus 44 Ag [Presence] in Specimen
        '17408-6',  # Human papilloma virus 45 Ag [Presence] in Specimen
        '17409-4',  # Human papilloma virus 5 Ag [Presence] in Specimen
        '17410-2',  # Human papilloma virus 51 Ag [Presence] in Specimen
        '17411-0',  # Human papilloma virus 6 Ag [Presence] in Specimen
        '17412-8',  # Human papilloma virus 6+11 Ag [Presence] in Specimen
        '17723-8',  # Treponema pallidum Ab [Presence] in Serum by Immobilization
        '17724-6',  # Treponema pallidum Ab [Units/volume] in Serum by Immunofluorescence
        '17725-3',  # Treponema pallidum Ab [Units/volume] in Serum by Latex agglutination
        '17726-1',  # Treponema pallidum IgG Ab [Presence] in Serum by Immunofluorescence
        '17727-9',  # Treponema pallidum IgG Ab [Units/volume] in Serum by Immunofluorescence
        '17728-7',  # Treponema pallidum IgM Ab [Units/volume] in Serum by Immunofluorescence
        '17729-5',  # Treponema pallidum IgM Ab [Presence] in Serum by Immunofluorescence
        '18478-8',  # Human papilloma virus 16+18 DNA [Presence] in Tissue by Probe
        '18479-6',  # Human papilloma virus 31+35+51 DNA [Presence] in Tissue by Probe
        '18480-4',  # Human papilloma virus 6+11 DNA [Presence] in Tissue by Probe
        '20507-0',  # Reagin Ab [Presence] in Serum by RPR
        '20508-8',  # Reagin Ab [Units/volume] in Serum by RPR
        '21414-8',  # Neisseria gonorrhoeae DNA [Presence] in Cervical mucus by NAA with probe detection
        '21415-5',  # Neisseria gonorrhoeae DNA [Presence] in Urethra by NAA with probe detection
        '21440-3',  # Human papilloma virus 16+18+31+33+35+45+51+52+56 DNA [Presence] in Cervix by Probe
        '21441-1',  # Human papilloma virus 6+11+42+43+44 DNA [Presence] in Cervix by Probe
        '22430-3',  # Neisseria gonorrhoeae Ab [Titer] in Serum
        '22434-5',  # Human papilloma virus Ab [Presence] in Tissue
        '22461-8',  # Reagin Ab [Presence] in Serum
        '22462-6',  # Reagin Ab [Units/volume] in Serum
        '22585-4',  # Treponema pallidum Ab [Units/volume] in Blood
        '22587-0',  # Treponema pallidum Ab [Presence] in Serum
        '22590-4',  # Treponema pallidum Ab [Titer] in Serum
        '22592-0',  # Treponema pallidum IgG Ab [Units/volume] in Serum
        '22594-6',  # Treponema pallidum IgM Ab [Units/volume] in Serum
        '24110-9',  # Treponema pallidum Ab [Presence] in Serum by Immunoassay
        '24111-7',  # Neisseria gonorrhoeae DNA [Presence] in Specimen by NAA with probe detection
        '24312-1',  # Treponema pallidum Ab [Presence] in Serum by Agglutination
        '26009-1',  # Treponema pallidum Ab [Titer] in Serum by Hemagglutination
        '29311-8',  # Neisseria gonorrhoeae Ag [Presence] in Specimen by Immunofluorescence
        '30167-1',  # Human papilloma virus 16+18+31+33+35+39+45+51+52+56+58+59+68 DNA [Presence] in Cervix by Probe with signal amplification
        '31147-2',  # Reagin Ab [Titer] in Serum by RPR
        '31905-3',  # Neisseria gonorrhoeae Ag [Presence] in Genital specimen
        '31906-1',  # Neisseria gonorrhoeae Ag [Presence] in Specimen
        '32047-3',  # Human papilloma virus 16+18+31+33+35+39+45+51+52+56+58+68+70 DNA [Presence] in Tissue by Probe
        '32198-4',  # Neisseria gonorrhoeae rRNA [Presence] in Cervix by Probe
        '32199-2',  # Neisseria gonorrhoeae rRNA [Presence] in Urethra by Probe
        '32704-9',  # Neisseria gonorrhoeae Ab [Presence] in Serum
        '32705-6',  # Neisseria gonorrhoeae DNA [Presence] in Vaginal fluid by NAA with probe detection
        '34147-9',  # Treponema pallidum IgG+IgM Ab [Presence] in Serum
        '34382-2',  # Treponema pallidum Ab [Titer] in Serum by Immunofluorescence
        '38372-9',  # Human papilloma virus 6+11+16+18+31+33+35+39+42+43+44+45+51+52+56+58+59+68 DNA [Presence] in Cervix by Probe with signal amplification
        '40679-3',  # Treponema pallidum IgG Ab [Presence] in Serum by Immunoblot
        '40680-1',  # Treponema pallidum IgM Ab [Presence] in Serum by Immunoblot
        '41122-3',  # Treponema pallidum Ab [Units/volume] in Specimen
        '42481-2',  # Human papilloma virus 6+11+42+43+44 DNA [Presence] in Cervix by Probe with signal amplification
        '43170-0',  # Human papilloma virus 31+33 DNA [Presence] in Tissue by Probe with signal amplification
        '43305-2',  # Neisseria gonorrhoeae rRNA [Presence] in Specimen by NAA with probe detection
        '43403-5',  # Neisseria gonorrhoeae DNA [Presence] in Specimen by Probe with signal amplification
        '44543-7',  # Human papilloma virus 31+33 Ag [Presence] in Specimen
        '44544-5',  # Human papilloma virus 35 Ag [Presence] in Specimen
        '44545-2',  # Human papilloma virus DNA [Presence] in Urethra by Probe
        '44546-0',  # Human papilloma virus DNA [Presence] in Vaginal fluid by Probe
        '44547-8',  # Human papilloma virus DNA [Presence] in Specimen by Probe with signal amplification
        '44548-6',  # Human papilloma virus DNA [Presence] in Urethra by Probe with signal amplification
        '44549-4',  # Human papilloma virus DNA [Presence] in Vaginal fluid by Probe with signal amplification
        '44550-2',  # Human papilloma virus DNA [Presence] in Cervix by Probe
        '44551-0',  # Human papilloma virus DNA [Presence] in Tissue by Probe
        '47236-5',  # Treponema pallidum IgG+IgM Ab [Presence] in Serum by Immunoassay
        '47237-3',  # Treponema pallidum IgM Ab [Presence] in Serum by Immunoassay
        '47238-1',  # Treponema pallidum IgG Ab [Presence] in Serum by Immunoassay
        '47387-6',  # Neisseria gonorrhoeae DNA [Presence] in Genital specimen by NAA with probe detection
        '48560-7',  # Human papilloma virus genotype [Identifier] in Specimen by NAA with probe detection
        '49891-5',  # Human papilloma virus DNA [Presence] in Specimen by NAA with probe detection
        '49896-4',  # Human papilloma virus 16+18+31+33+35+39+45+51+52+56+58+59+68 DNA [Presence] in Specimen by NAA with probe detection
        '5028-6',  # Neisseria gonorrhoeae rRNA [Presence] in Specimen by Probe
        '50326-8',  # Neisseria gonorrhoeae DNA [Presence] in Specimen from Donor by NAA with probe detection
        '50388-8',  # Neisseria gonorrhoeae rRNA [Presence] in Cervix by NAA with probe detection
        '50412-6',  # Neisseria gonorrhoeae rRNA [Presence] in Specimen from Donor by NAA with probe detection
        '50690-7',  # Reagin Ab [Titer] in Serum by VDRL
        '51838-1',  # Treponema pallidum IgG Ab [Units/volume] in Serum by Immunoassay
        '51839-9',  # Treponema pallidum IgM Ab [Units/volume] in Serum by Immunoassay
        '5291-0',  # Reagin Ab [Units/volume] in Serum by VDRL
        '5292-8',  # Reagin Ab [Presence] in Serum by VDRL
        '53762-1',  # Neisseria gonorrhoeae IgG Ab [Titer] in Serum
        '53879-3',  # Neisseria gonorrhoeae rRNA [Presence] in Vaginal fluid by NAA with probe detection
        '5392-6',  # Treponema pallidum Ab [Units/volume] in Serum by Immobilization
        '53927-0',  # Neisseria gonorrhoeae rRNA [Presence] in Urethra by NAA with probe detection
        '5393-4',  # Treponema pallidum Ab [Presence] in Serum by Immunofluorescence
        '5394-2',  # Treponema pallidum Ab [Titer] in Serum by Latex agglutination
        '55298-4',  # Human papilloma virus 6+11+42+43+44 DNA [Presence] in Urethra by Probe with signal amplification
        '55299-2',  # Human papilloma virus 6+11+42+43+44 DNA [Presence] in Specimen by Probe with signal amplification
        '57032-5',  # Treponema pallidum Ab [Presence] in Serum by Immunoblot
        '59263-4',  # Human papilloma virus 16 DNA [Presence] in Cervix by Probe with signal amplification
        '59264-2',  # Human papilloma virus 18 DNA [Presence] in Cervix by Probe with signal amplification
        '59420-0',  # Human papilloma virus 16+18+31+33+35+39+45+51+52+56+58+59+66+68 DNA [Presence] in Cervix by Probe with signal amplification
        '61372-9',  # Human papilloma virus 16 DNA [Presence] in Specimen by NAA with probe detection
        '61373-7',  # Human papilloma virus 18 DNA [Presence] in Specimen by NAA with probe detection
        '61374-5',  # Human papilloma virus 26 DNA [Presence] in Specimen by NAA with probe detection
        '61375-2',  # Human papilloma virus 31 DNA [Presence] in Specimen by NAA with probe detection
        '61376-0',  # Human papilloma virus 33 DNA [Presence] in Specimen by NAA with probe detection
        '61377-8',  # Human papilloma virus 35 DNA [Presence] in Specimen by NAA with probe detection
        '61378-6',  # Human papilloma virus 39 DNA [Presence] in Specimen by NAA with probe detection
        '61379-4',  # Human papilloma virus 44 DNA [Presence] in Specimen by NAA with probe detection
        '61380-2',  # Human papilloma virus 45 DNA [Presence] in Specimen by NAA with probe detection
        '61381-0',  # Human papilloma virus 51 DNA [Presence] in Specimen by NAA with probe detection
        '61382-8',  # Human papilloma virus 52 DNA [Presence] in Specimen by NAA with probe detection
        '61383-6',  # Human papilloma virus 53 DNA [Presence] in Specimen by NAA with probe detection
        '61384-4',  # Human papilloma virus 58 DNA [Presence] in Specimen by NAA with probe detection
        '61385-1',  # Human papilloma virus 59 DNA [Presence] in Specimen by NAA with probe detection
        '61386-9',  # Human papilloma virus 66 DNA [Presence] in Specimen by NAA with probe detection
        '61387-7',  # Human papilloma virus 67 DNA [Presence] in Specimen by NAA with probe detection
        '61388-5',  # Human papilloma virus 68 DNA [Presence] in Specimen by NAA with probe detection
        '61389-3',  # Human papilloma virus 69 DNA [Presence] in Specimen by NAA with probe detection
        '61390-1',  # Human papilloma virus 70 DNA [Presence] in Specimen by NAA with probe detection
        '61391-9',  # Human papilloma virus 73 DNA [Presence] in Specimen by NAA with probe detection
        '61392-7',  # Human papilloma virus 82 DNA [Presence] in Specimen by NAA with probe detection
        '61393-5',  # Human papilloma virus 6 DNA [Presence] in Specimen by NAA with probe detection
        '61394-3',  # Human papilloma virus 11 DNA [Presence] in Specimen by NAA with probe detection
        '61395-0',  # Human papilloma virus 42 DNA [Presence] in Specimen by NAA with probe detection
        '61396-8',  # Human papilloma virus 56 DNA [Presence] in Specimen by NAA with probe detection
        '63464-2',  # Treponema pallidum Ab [Units/volume] in Serum by Immunoassay
        '6487-3',  # Neisseria gonorrhoeae Ag [Presence] in Genital specimen by Immunoassay
        '6488-1',  # Neisseria gonorrhoeae Ag [Presence] in Genital specimen by Immunofluorescence
        '6489-9',  # Neisseria gonorrhoeae Ag [Presence] in Genital specimen by Latex agglutination
        '6490-7',  # Neisseria gonorrhoeae Ag [Presence] in Urethra
        '6510-2',  # Human papilloma virus Ab [Presence] in Genital specimen by Immunoassay
        '6511-0',  # Human papilloma virus Ab [Presence] in Genital specimen by Immunoblot
        '6512-8',  # Human papilloma virus Ab [Presence] in Tissue by Immunoassay
        '6513-6',  # Human papilloma virus Ab [Presence] in Tissue by Immunoblot
        '6514-4',  # Human papilloma virus rRNA [Presence] in Genital specimen by NAA with probe detection
        '6515-1',  # Human papilloma virus rRNA [Presence] in Tissue by NAA with probe detection
        '6516-9',  # Human papilloma virus rRNA [Presence] in Specimen by NAA with probe detection
        '6561-5',  # Treponema pallidum IgG Ab [Presence] in Serum
        '6562-3',  # Treponema pallidum IgM Ab [Presence] in Serum
        '688-2',  # Neisseria gonorrhoeae [Presence] in Cervix by Organism specific culture
        '690-8',  # Neisseria gonorrhoeae [Presence] in Endometrium by Organism specific culture
        '69002-4',  # Human papilloma virus E6+E7 mRNA [Presence] in Cervix by NAA with probe detection
        '691-6',  # Neisseria gonorrhoeae [Presence] in Genital specimen by Organism specific culture
        '692-4',  # Neisseria gonorrhoeae [Presence] in Genital lochia by Organism specific culture
        '693-2',  # Neisseria gonorrhoeae [Presence] in Vaginal fluid by Organism specific culture
        '697-3',  # Neisseria gonorrhoeae [Presence] in Urethra by Organism specific culture
        '698-1',  # Neisseria gonorrhoeae [Presence] in Specimen by Organism specific culture
        '70061-7',  # Human papilloma virus 16 and 18 DNA [Presence] in Specimen by NAA with probe detection
        '71431-1',  # Human papilloma virus 31+33+35+39+45+51+52+56+58+59+66+68 DNA [Presence] in Cervix by NAA with probe detection
        '71793-4',  # Treponema pallidum Ab [Titer] in Serum or Plasma by Agglutination
        '73732-0',  # Human papilloma virus 26+31+33+35+39+45+51+52+53+56+58+59+66+68+73+82 DNA [Presence] in Genital specimen by NAA with probe detection
        '73959-9',  # Human papilloma virus 16+18+31+33+35+39+45+51+52+56+58+66 DNA [Presence] in Tissue by Probe
        '75694-0',  # Human papilloma virus 18+45 E6+E7 mRNA [Presence] in Cervix by NAA with probe detection
        '77375-4',  # Human papilloma virus 31+33+35+39+45+51+52+56+58+59+66+68 DNA [Presence] in Specimen by NAA with probe detection
        '77399-4',  # Human papilloma virus 16 DNA [Presence] in Cervix by NAA with probe detection
        '77400-0',  # Human papilloma virus 18 DNA [Presence] in Cervix by NAA with probe detection
        '7975-6',  # Human papilloma virus Ab [Presence] in Genital specimen
        '8041-6',  # Treponema pallidum Ab [Presence] in Serum by Hemagglutination
        '82354-2',  # Human papilloma virus 16 and 18+45 E6+E7 mRNA [Identifier] in Cervix by NAA with probe detection
        '82456-5',  # Human papilloma virus 16 E6+E7 mRNA [Presence] in Cervix by NAA with probe detection
        '82675-0',  # Human papilloma virus 16+18+31+33+35+39+45+51+52+56+58+59+66+68 DNA [Presence] in Cervix by NAA with probe detection
        '86560-0',  # Human papilloma virus 16 DNA [Presence] in Tissue by NAA with probe detection
        '86561-8',  # Human papilloma virus 16+18+31+33+35+39+45+51+52+56+58+59+68 DNA [Presence] in Tissue by NAA with probe detection
        '86562-6',  # Human papilloma virus 18 DNA [Presence] in Tissue by NAA with probe detection
        '86563-4',  # Human papilloma virus 31+33+35+39+45+51+52+56+58+59+66+68 DNA [Presence] in Tissue by NAA with probe detection
        '86564-2',  # Human papilloma virus DNA [Presence] in Tissue by NAA with probe detection
        '91073-7',  # Human papilloma virus genotype [Identifier] in Tissue by NAA with probe detection
        '91851-6',  # Human papilloma virus genotype [Identifier] in Genital specimen by NAA with probe detection
        '91852-4',  # Human papilloma virus DNA [Presence] in Genital specimen by NAA with probe detection
        '91854-0',  # Human papilloma virus 18 DNA [Presence] in Genital specimen by NAA with probe detection
        '91855-7',  # Human papilloma virus 16+18+31+33+35+39+45+51+52+56+58+59+68 DNA [Presence] in Genital specimen by NAA with probe detection
        '91856-5',  # Human papilloma virus 16 DNA [Presence] in Genital specimen by NAA with probe detection
        '93777-1',  # Human papilloma virus 16+18 E6+E7 mRNA [Presence] in Tissue by Probe
        '93778-9',  # Human papilloma virus 6+11 E6+E7 mRNA [Presence] in Tissue by Probe
        '94425-6',  # Human papilloma virus E6+E7 mRNA [Presence] in Specimen by NAA with probe detection
        '95533-6',  # Human papilloma virus 56+59+66 DNA [Presence] in Cervix by NAA with probe detection
        '95534-4',  # Human papilloma virus 52 DNA [Presence] in Cervix by NAA with probe detection
        '95535-1',  # Human papilloma virus 51 DNA [Presence] in Cervix by NAA with probe detection
        '95536-9',  # Human papilloma virus 45 DNA [Presence] in Cervix by NAA with probe detection
        '95537-7',  # Human papilloma virus 35+39+68 DNA [Presence] in Cervix by NAA with probe detection
        '95538-5',  # Human papilloma virus 33+58 DNA [Presence] in Cervix by NAA with probe detection
        '95539-3',  # Human papilloma virus 31 DNA [Presence] in Cervix by NAA with probe detection
        '9568-7',  # Neisseria gonorrhoeae Ab [Titer] in Serum by Complement fixation
        '95938-7',  # Fetal Cerebellum Diameter transverse percentile per estimated gestational age
        '95939-5',  # Fetal Crown Rump length percentile per estimated gestational age
        '95940-3',  # Fetal Head Diameter.biparietal percentile per estimated gestational age
        '96124-3',  # Fetal femur diaphysis length percentile per estimated gestational age
        '96125-0',  # Fetal Abdomen Circumference Per estimated gestational age
        '96426-2',  # Fetal Nasal bone
        '96599-6',  # Neisseria gonorrhoeae DNA [Presence] in Cervix by NAA with probe detection
    }


class PregnancyTest(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for pregnancy tests.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent tests for pregnancy including in urine, serum or plasma.

    **Exclusion Criteria:** Excludes concepts that represent an order only.

    ** Used in:** CMS153v10
    """

    VALUE_SET_NAME = 'Pregnancy Test'
    OID = '2.16.840.1.113883.3.464.1003.111.12.1011'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '19080-1',  # Choriogonadotropin [Units/volume] in Serum or Plasma
        '19180-9',  # Choriogonadotropin.beta subunit free [Units/volume] in Serum or Plasma
        '20415-6',  # Choriogonadotropin.beta subunit [Units/volume] in Serum or Plasma by Immunoassay (EIA) 3rd IS
        '20994-0',  # Choriogonadotropin [Interpretation] in Serum or Plasma
        '2106-3',  # Choriogonadotropin (pregnancy test) [Presence] in Urine
        '2107-1',  # Choriogonadotropin [Moles/volume] in Urine
        '2110-5',  # Choriogonadotropin.beta subunit (pregnancy test) [Presence] in Serum or Plasma
        '2111-3',  # Choriogonadotropin.beta subunit [Moles/volume] in Serum or Plasma
        '2112-1',  # Choriogonadotropin.beta subunit (pregnancy test) [Presence] in Urine
        '2113-9',  # Choriogonadotropin.beta subunit [Mass/time] in 24 hour Urine
        '2114-7',  # Choriogonadotropin.beta subunit [Moles/volume] in Urine
        '2115-4',  # Choriogonadotropin.beta subunit free [Moles/volume] in Serum or Plasma
        '2118-8',  # Choriogonadotropin (pregnancy test) [Presence] in Serum or Plasma
        '2119-6',  # Choriogonadotropin [Moles/volume] in Serum or Plasma
        '21198-7',  # Choriogonadotropin.beta subunit [Units/volume] in Serum or Plasma
        '25372-4',  # Choriogonadotropin [Units/volume] in Urine
        '25373-2',  # Choriogonadotropin.beta subunit free [Mass/volume] in Serum or Plasma
        '34670-0',  # Choriogonadotropin [Mass/volume] in Serum or Plasma
        '45194-8',  # Choriogonadotropin.intact+Beta subunit [Units/volume] in Serum or Plasma
        '55869-2',  # Choriogonadotropin.beta subunit [Mass/volume] in Serum or Plasma
        '55870-0',  # Choriogonadotropin [Units/volume] in Cord blood
        '56497-1',  # Choriogonadotropin.beta subunit [Units] in 24 hour Urine
        '80384-1',  # Choriogonadotropin (pregnancy test) [Presence] in Urine by Rapid immunoassay
        '80385-8',  # Choriogonadotropin (pregnancy test) [Presence] in Serum by Rapid immunoassay
        '83086-9',  # Choriogonadotropin [Units/volume] in Serum or Plasma by Immunoassay
        '93769-8',  # Choriogonadotropin.intact+Beta subunit [Mass/volume] in Serum or Plasma
    }


class LaboratoryTestsForHypertension(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for lab  tests that are commonly performed on patients with hypertension.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent concepts for lab tests that are commonly performed on patients with hypertension.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS22v10
    """

    VALUE_SET_NAME = 'Laboratory Tests for Hypertension'
    OID = '2.16.840.1.113883.3.600.1482'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '24320-4',  # Basic metabolic 1998 panel - Serum or Plasma
        '24321-2',  # Basic metabolic 2000 panel - Serum or Plasma
        '24323-8',  # Comprehensive metabolic 2000 panel - Serum or Plasma
        '24356-8',  # Urinalysis complete panel - Urine
        '24357-6',  # Urinalysis macro (dipstick) panel - Urine
        '24362-6',  # Renal function 2000 panel - Serum or Plasma
        '2888-6',  # Protein [Mass/volume] in Urine
        '57021-8',  # CBC W Auto Differential panel - Blood
        '57782-5',  # CBC W Ordered Manual Differential panel - Blood
        '58410-2',  # CBC panel - Blood by Automated count
    }


class LdlCholesterol(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for laboratory tests commonly used for low-density lipoproteins (LDL) cholesterol measurement.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent a laboratory test measuring low-density lipoprotein  cholesterol (LDL-C) using the specimen type of serum or plasma based on a measurement scale of mass per volume.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS347v5
    """

    VALUE_SET_NAME = 'LDL Cholesterol'
    OID = '2.16.840.1.113883.3.526.3.1573'
    DEFINITION_VERSION = '20200307'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '13457-7',  # Cholesterol in LDL [Mass/volume] in Serum or Plasma by calculation
        '18261-8',  # Cholesterol in LDL [Mass/volume] in Serum or Plasma ultracentrifugate
        '18262-6',  # Cholesterol in LDL [Mass/volume] in Serum or Plasma by Direct assay
        '2089-1',  # Cholesterol in LDL [Mass/volume] in Serum or Plasma
        '43394-6',  # Cholesterol in LDL acylated [Mass/volume] in Serum or Plasma
        '49132-4',  # Cholesterol in LDL [Mass/volume] in Serum or Plasma by Electrophoresis
        '50193-2',  # Cholesterol in LDL.narrow density [Mass/volume] in Serum or Plasma
        '55440-2',  # Cholesterol.in LDL (real) [Mass/volume] in Serum or Plasma by VAP
        '86911-5',  # Cholesterol in LDL goal [Mass/volume] Serum or Plasma
        '90364-1',  # Cholesterol.in LDL.small dense [Mass/volume] in Serum by Immunoassay
        '91105-7',  # Cholesterol in LDL 1 [Mass/volume] in Serum or Plasma
        '91106-5',  # Cholesterol in LDL 2 [Mass/volume] in Serum or Plasma
        '91107-3',  # Cholesterol in LDL 3 [Mass/volume] in Serum or Plasma
        '91108-1',  # Cholesterol in LDL 4 [Mass/volume] in Serum or Plasma
        '91109-9',  # Cholesterol in LDL 5 [Mass/volume] in Serum or Plasma
        '91110-7',  # Cholesterol in LDL 6 [Mass/volume] in Serum or Plasma
        '91111-5',  # Cholesterol in LDL 7 [Mass/volume] in Serum or Plasma
        '96259-7',  # Cholesterol in LDL [Mass/volume] in Serum or Plasma by Calculated by Martin-Hopkins
        '96597-0',  # Cholesterol in LDL [Mass/volume] in DBS by Direct assay
    }


class HumanImmunodeficiencyVirusHivLaboratoryTestCodesAbAndAg(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for laboratory tests used for Human Immunodeficiency Virus (HIV) screening.

    **Data Element Scope:** This value set may use a model element related to Laboratory Test.

    **Inclusion Criteria:** Includes concepts that represent a laboratory test for HIV-1 and HIV-2 antibodies or antigens.

    **Exclusion Criteria:** Excludes concepts that represent tests and procedures that might be associated with HIV infection but not used for screening or testing to establish an HIV diagnosis such as home/self HIV testing, HIV genotyping tests, HIV RNA tests, HIV cultures, clinical codes used to document care provided to HIV-infected patients.

    ** Used in:** CMS349v4
    """

    VALUE_SET_NAME = 'Human Immunodeficiency Virus (HIV) Laboratory Test Codes (Ab and Ag)'
    OID = '2.16.840.1.113762.1.4.1056.50'
    DEFINITION_VERSION = '20210226'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    LOINC = {
        '10901-7',  # HIV 2 gp125 Ab [Presence] in Serum by Immunoblot
        '10902-5',  # HIV 2 gp36 Ab [Presence] in Serum by Immunoblot
        '11078-3',  # HIV 2 gp80 Ab [Presence] in Serum by Immunoblot
        '11079-1',  # HIV 2 p26 Ab [Presence] in Serum by Immunoblot
        '11080-9',  # HIV 2 p53 Ab [Presence] in Serum by Immunoblot
        '11081-7',  # HIV 2 p56 Ab [Presence] in Serum by Immunoblot
        '11082-5',  # HIV 2 p68 Ab [Presence] in Serum by Immunoblot
        '12855-3',  # HIV 1 p23 Ab [Presence] in Serum by Immunoblot
        '12856-1',  # HIV 1 p65 Ab [Presence] in Serum by Immunoblot
        '12857-9',  # HIV 1 p28 Ab [Presence] in Serum by Immunoblot
        '12858-7',  # HIV 1 p32 Ab [Presence] in Serum by Immunoblot
        '12859-5',  # HIV 1 p18 Ab [Presence] in Serum by Immunoblot
        '12870-2',  # HIV 1 gp34 Ab [Presence] in Serum by Immunoblot
        '12871-0',  # HIV 1 p26 Ab [Presence] in Serum by Immunoblot
        '12872-8',  # HIV 1 p15 Ab [Presence] in Serum by Immunoblot
        '12875-1',  # HIV 1 p64 Ab [Presence] in Serum by Immunoblot
        '12876-9',  # HIV 1 p53 Ab [Presence] in Serum by Immunoblot
        '12893-4',  # HIV 1 gp105 Ab [Presence] in Serum by Immunoblot
        '12894-2',  # HIV 1 p68 Ab [Presence] in Serum by Immunoblot
        '12895-9',  # HIV 1 p58 Ab [Presence] in Serum by Immunoblot
        '13499-9',  # HIV 1 Ab band pattern [Interpretation] in Serum by Immunoblot
        '13920-4',  # HIV 2 p41 Ab [Presence] in Serum by Immunoblot
        '14092-1',  # HIV 1 Ab [Presence] in Serum by Immunofluorescence
        '14126-7',  # HIV 1 gp120+gp160 Ab [Presence] in Serum by Immunoblot
        '16132-3',  # HIV 1 p15+p18 Ab [Presence] in Serum by Immunoblot
        '16974-8',  # HIV 1 Ab [Presence] in Cerebral spinal fluid
        '16975-5',  # HIV 1 IgG Ab [Presence] in Serum
        '16976-3',  # HIV 1 Ag [Units/volume] in Serum
        '16977-1',  # HIV 1 Ag [Presence] in Cerebral spinal fluid
        '16978-9',  # HIV 1 p24 Ab [Units/volume] in Serum
        '16979-7',  # HIV 1 p24 Ag [Units/volume] in Cerebral spinal fluid
        '18396-2',  # HIV 1 p24 Ag [Presence] in Serum or Plasma by Immunoassay
        '19110-6',  # HIV 1 gp41+gp43 Ab [Presence] in Serum by Immunoblot
        '21007-0',  # HIV 1 Ab [Presence] in Serum from Donor
        '21331-4',  # HIV 1 p24 Ab [Titer] in Serum by Immunoblot
        '21332-2',  # HIV 1 p41 Ab [Titer] in Serum by Immunoblot
        '21334-8',  # HIV 2 gp105 Ab [Presence] in Serum by Immunoblot
        '21335-5',  # HIV 2 gp120 Ab [Presence] in Serum by Immunoblot
        '21336-3',  # HIV 2 gp15 Ab [Presence] in Serum by Immunoblot
        '21337-1',  # HIV 2 gp34 Ab [Presence] in Serum by Immunoblot
        '21338-9',  # HIV 2 p31 Ab [Presence] in Serum by Immunoblot
        '21339-7',  # HIV 2 p55 Ab [Presence] in Serum by Immunoblot
        '21340-5',  # HIV 2 p58 Ab [Presence] in Serum by Immunoblot
        '22356-0',  # HIV 1 Ab [Units/volume] in Serum
        '22357-8',  # HIV 1+2 Ab [Units/volume] in Serum
        '22358-6',  # HIV 2 Ab [Units/volume] in Serum
        '24012-7',  # HIV 1 Ag [Presence] in Serum
        '28004-0',  # HIV 1 IgG Ab [Presence] in Cerebral spinal fluid by Immunoblot
        '28052-9',  # HIV 1 IgG band pattern [Interpretation] in Cerebral spinal fluid by Immunoblot
        '29327-4',  # HIV 1 Ab [Presence] in Body fluid
        '29893-5',  # HIV 1 Ab [Presence] in Serum or Plasma by Immunoassay
        '30361-0',  # HIV 2 Ab [Presence] in Serum or Plasma by Immunoassay
        '31072-2',  # HIV 1 p41 Ab [Presence] in Serum by Immunoblot
        '31073-0',  # HIV 2 Ab band pattern [Interpretation] in Serum by Immunoblot
        '31201-7',  # HIV 1+2 Ab [Presence] in Serum or Plasma by Immunoassay
        '31430-2',  # HIV 1 IgG Ab [Presence] in Cerebral spinal fluid
        '32571-2',  # HIV 1 Ab [Presence] in Urine by Immunoblot
        '32602-5',  # HIV 1+2 Ab [Presence] in Cerebral spinal fluid
        '32827-8',  # HIV 1 p17+p18 Ab [Presence] in Serum by Immunoblot
        '32842-7',  # HIV 1 p17+p18 Ab [Presence] in Serum
        '33508-3',  # HIV 1 p65+p66 Ab [Presence] in Serum by Immunoblot
        '33660-2',  # HIV 1 p24 Ag [Presence] in Serum or Plasma by Neutralization test
        '33806-1',  # HIV 2 IgG Ab [Presence] in Serum by Immunoblot
        '33807-9',  # HIV 2 IgG Ab [Presence] in Serum
        '33866-5',  # HIV 1 Ab [Presence] in Capillary blood by Immunoassay
        '34591-8',  # HIV 1 Ab [Presence] in Body fluid by Immunoassay
        '34592-6',  # HIV 1 Ab [Presence] in Body fluid by Immunoblot
        '35437-3',  # HIV 1 Ab [Presence] in Saliva (oral fluid) by Immunoassay
        '35438-1',  # HIV 1 Ab [Units/volume] in Saliva (oral fluid) by Immunoassay
        '35439-9',  # HIV 1 Ab [Presence] in Saliva (oral fluid) by Immunoblot
        '35440-7',  # HIV 1 gp160 Ab [Presence] in Saliva (oral fluid) by Immunoblot
        '35441-5',  # HIV 1 gp120 Ab [Presence] in Saliva (oral fluid) by Immunoblot
        '35442-3',  # HIV 1 p66 Ab [Presence] in Saliva (oral fluid) by Immunoblot
        '35443-1',  # HIV 1 p65 Ab [Presence] in Saliva (oral fluid) by Immunoblot
        '35444-9',  # HIV 1 p55 Ab [Presence] in Saliva (oral fluid) by Immunoblot
        '35445-6',  # HIV 1 p51 Ab [Presence] in Saliva (oral fluid) by Immunoblot
        '35446-4',  # HIV 1 gp41 Ab [Presence] in Saliva (oral fluid) by Immunoblot
        '35447-2',  # HIV 1 p31 Ab [Presence] in Saliva (oral fluid) by Immunoblot
        '35448-0',  # HIV 1 p24 Ab [Presence] in Saliva (oral fluid) by Immunoblot
        '35449-8',  # HIV 1 p17 Ab [Presence] in Saliva (oral fluid) by Immunoblot
        '35450-6',  # HIV 1 p18 Ab [Presence] in Saliva (oral fluid) by Immunoblot
        '35452-2',  # HIV 1 gp40 Ab [Presence] in Serum by Immunoblot
        '35564-4',  # HIV 1 p31+p32 Ab [Presence] in Serum by Immunoblot
        '35565-1',  # HIV 1 p40 Ab [Presence] in Serum by Immunoblot
        '40437-6',  # HIV 1 p24 Ab [Presence] in Serum or Plasma by Immunoassay
        '40438-4',  # HIV 1 gp41 Ab [Presence] in Serum or Plasma by Immunoassay
        '40439-2',  # HIV 1 gp120+gp160 Ab [Presence] in Serum or Plasma by Immunoassay
        '40732-0',  # HIV 1 IgG Ab [Presence] in Serum by Immunoblot
        '40733-8',  # HIV 1+2 IgG Ab [Presence] in Serum or Plasma by Immunoassay
        '41143-9',  # HIV 1 Ab [Units/volume] in Saliva (oral fluid)
        '41144-7',  # HIV 1 Ab [Presence] in Saliva (oral fluid)
        '41145-4',  # HIV 1 Ab [Presence] in Capillary blood
        '41290-8',  # HIV 1+2 IgM Ab [Units/volume] in Serum or Plasma by Immunoassay
        '42339-2',  # HIV 1 p24 Ag [Mass/volume] in Serum or Plasma by Immunoassay
        '42600-7',  # HIV 1+2 Ab [Presence] in Specimen by Immunoassay
        '42627-0',  # HIV 1 Ab [Presence] in Cerebral spinal fluid by Immunofluorescence
        '42768-2',  # HIV 1 and 2 Ab [Interpretation] in Serum Narrative
        '43008-2',  # HIV 1+2 IgM Ab [Units/volume] in Serum
        '43009-0',  # HIV 1+2 IgG Ab [Presence] in Serum
        '43010-8',  # HIV 1+2 Ab [Presence] in Specimen
        '43011-6',  # HIV 1 p24 Ab [Presence] in Serum
        '43012-4',  # HIV 1 gp41 Ab [Presence] in Serum
        '43013-2',  # HIV 1 gp120+gp160 Ab [Presence] in Serum
        '43185-8',  # HIV 1 and 2 Ab band pattern [Interpretation] in Serum by Immunoblot
        '43599-0',  # HIV 1 Ab [Units/volume] in Serum by Immunofluorescence
        '44531-2',  # HIV 1 Ag [Presence] in Serum from Donor
        '44532-0',  # HIV 1 gp120 Ab [Presence] in Serum
        '44533-8',  # HIV 1+2 Ab [Presence] in Serum from Donor
        '44607-0',  # HIV 1 [Interpretation] in Serum or Plasma by Immunoassay
        '44872-0',  # HIV 1 p24 Ag [Presence] in Serum or Plasma from Donor by Immunoassay
        '44873-8',  # HIV 1+2 Ab [Presence] in Serum by Immunoblot
        '45212-8',  # HIV 2 p31+p34 Ab [Presence] in Serum by Immunoblot
        '47029-4',  # HIV 2 Ab [Presence] in Cerebral spinal fluid by Immunoblot
        '48345-3',  # HIV 1+O+2 Ab [Presence] in Serum or Plasma
        '48346-1',  # HIV 1+O+2 Ab [Units/volume] in Serum or Plasma
        '49483-1',  # HIV 1 [Interpretation] in Serum or Plasma by Immunoassay Narrative
        '49580-4',  # HIV 1+2 Ab [Presence] in Specimen by Rapid immunoassay
        '49718-0',  # HIV 1 p24 Ag [Presence] in Cerebral spinal fluid
        '49905-3',  # HIV 1 Ab [Presence] in Specimen by Rapid immunoassay
        '49965-7',  # Deprecated HIV 1 Ab/HIV 2 Ab Ab [Ratio] in Serum by Immunoblot (IB)
        '51786-2',  # HIV 2 Ab Signal/Cutoff in Serum or Plasma by Immunoassay
        '51866-2',  # Deprecated HIV 1 Ab+Ag [Presence] in Serum
        '5220-9',  # HIV 1 Ab [Units/volume] in Serum or Plasma by Immunoassay
        '5221-7',  # HIV 1 Ab [Presence] in Serum or Plasma by Immunoblot
        '5222-5',  # HIV 1 Ag [Presence] in Serum or Plasma by Immunoassay
        '5223-3',  # HIV 1+2 Ab [Units/volume] in Serum or Plasma by Immunoassay
        '5224-1',  # HIV 2 Ab [Units/volume] in Serum or Plasma by Immunoassay
        '5225-8',  # HIV 2 Ab [Presence] in Serum by Immunoblot
        '53379-4',  # HIV 1 Ab [Presence] in Specimen
        '53601-1',  # HIV 1 p24 Ag [Units/volume] in Serum or Plasma by Immunoassay
        '54086-4',  # HIV 1+2 IgG Ab [Presence] in DBS
        '56888-1',  # HIV 1+2 Ab+HIV1 p24 Ag [Presence] in Serum or Plasma by Immunoassay
        '57974-8',  # HIV 1 Ab [Presence] in Cerebral spinal fluid by Immunoblot
        '57975-5',  # HIV 1+O+2 Ab [Presence] in Body fluid
        '57976-3',  # HIV 2 gp140 Ab [Presence] in Serum by Immunoblot
        '57977-1',  # HIV 2 p16 Ab [Presence] in Serum by Immunoblot
        '57978-9',  # HIV 2 p34 Ab [Presence] in Serum by Immunoblot
        '58900-2',  # HIV 1+2 Ab+HIV1 p24 Ag [Units/volume] in Serum or Plasma by Immunoassay
        '62456-9',  # HIV 2 p15 Ab [Presence] in Serum by Immunoblot
        '68961-2',  # HIV 1 Ab [Presence] in Serum, Plasma or Blood by Rapid immunoassay
        '69668-2',  # HIV 1 and 2 Ab [Identifier] in Serum or Plasma by Rapid immunoassay
        '73905-2',  # HIV 1+2 IgG Ab [Presence] in Serum or Plasma by Rapid immunoassay
        '73906-0',  # HIV 1+2 IgG Ab [Presence] in Blood by Rapid immunoassay
        '75622-1',  # HIV 1 and 2 tests - Meaningful Use set
        '75666-8',  # HIV 1+2 Ab and HIV1 p24 Ag [Identifier] in Serum, Plasma or Blood by Rapid immunoassay
        '77685-6',  # HIV 1 and 2 IgG Ab [Identifier] in Serum or Plasma by Immunoblot
        '7917-8',  # HIV 1 Ab [Presence] in Serum
        '7918-6',  # HIV 1+2 Ab [Presence] in Serum
        '7919-4',  # HIV 2 Ab [Presence] in Serum
        '80203-3',  # HIV 1 and 2 Ab [Identifier] in Serum, Plasma or Blood by Rapid immunoassay
        '80387-4',  # HIV 1+2 Ab [Presence] in Serum, Plasma or Blood by Rapid immunoassay
        '81641-3',  # HIV 2 Ab [Presence] in Serum, Plasma or Blood by Rapid immunoassay
        '83101-6',  # HIV 1+2 Ab and HIV1 p24 Ag panel - Serum or Plasma by Immunoassay
        '85037-0',  # HIV 1 and 2 Ab and HIV 1 p24 Ag panel - Serum or Plasma by Immunoassay
        '85686-4',  # HIV 1 Ab [Presence] in Serum, Plasma or Blood by Immunofluorescence
        '86233-4',  # HIV 1 Ab [Presence] in Serum, Plasma or Blood by Immunoblot
        '86657-4',  # HIV 1 and 2 tests - FPAR 2.0 set
        '89365-1',  # HIV 1 and 2 Ab panel - Serum, Plasma or Blood by Rapid immunoassay
        '89374-3',  # HIV 1 Ab [Presence] in Specimen by Immunoassay
        '9660-2',  # HIV 1 gp160 Ab [Presence] in Serum by Immunoblot
        '9661-0',  # HIV 1 gp120 Ab [Presence] in Serum by Immunoblot
        '9662-8',  # HIV 1 gp41 Ab [Presence] in Serum by Immunoblot
        '9663-6',  # HIV 1 p17 Ab [Presence] in Serum by Immunoblot
        '9664-4',  # HIV 1 p24 Ab [Presence] in Serum by Immunoblot
        '9665-1',  # HIV 1 p24 Ag [Units/volume] in Serum
        '9666-9',  # HIV 1 p31 Ab [Presence] in Serum by Immunoblot
        '9667-7',  # HIV 1 p51 Ab [Presence] in Serum by Immunoblot
        '9668-5',  # HIV 1 p55 Ab [Presence] in Serum by Immunoblot
        '9669-3',  # HIV 1 p66 Ab [Presence] in Serum by Immunoblot
        '9821-0',  # HIV 1 p24 Ag [Presence] in Serum
    }




from canvas_workflow_kit import events
from canvas_workflow_kit.protocol import ClinicalQualityMeasure
from canvas_workflow_kit.recommendation import TaskRecommendation
# flake8: noqa
from canvas_workflow_kit.value_set.v2018 import (
    AcetaminophenAspirinDiphenhydramine, AcetaminophenBrompheniramine, AcetaminophenButalbital,
    AcetaminophenButalbitalCaffeine, AcetaminophenButalbitalCaffeineCodeine,
    AcetaminophenChlorpheniramine, AcetaminophenChlorpheniramineDextromethorphan,
    AcetaminophenChlorpheniramineDextromethorphanGuaifenesin,
    AcetaminophenChlorpheniramineDextromethorphanPhenylephrine,
    AcetaminophenChlorpheniramineDextromethorphanPseudoephedrine,
    AcetaminophenChlorpheniraminePhenylephrine, AcetaminophenChlorpheniraminePseudoephedrine,
    AcetaminophenDexbrompheniramine, AcetaminophenDextromethorphanDiphenhydramine,
    AcetaminophenDextromethorphanDiphenhydraminePhenylephrine,
    AcetaminophenDextromethorphanDoxylamine, AcetaminophenDextromethorphanDoxylaminePhenylephrine,
    AcetaminophenDextromethorphanDoxylaminePseudoephedrine, AcetaminophenDiphenhydramine,
    AcetaminophenDiphenhydraminePhenylephrine, AcetaminophenDiphenhydraminePseudoephedrine,
    AcetaminophenDoxylaminePhenylephrine, AmitriptylineChlordiazepoxide,
    AmitriptylineHydrochloride, AmitriptylinePerphenazine, Amobarbital, AnnualWellnessVisit,
    AntiInfectivesOther, AspirinButalbitalCaffeine, AspirinButalbitalCaffeineCodeine,
    AspirinCaffeineOrphenadrine, AspirinCarisoprodol, AspirinCarisoprodolCodeine,
    AspirinChlorpheniraminePhenylephrine, AspirinDiphenhydramineCitrate, AspirinMeprobamate,
    AtropineChlorpheniramineHyoscyaminePseudoephedrineScopolamine,
    AtropineHyoscyaminePhenobarbitalScopolamine, BazedoxifeneConjugatedEstrogens, Benztropine,
    Brompheniramine, BrompheniramineChlophedianolPhenylephrine,
    BrompheniramineChlophedianolPseudoephedrine, BrompheniramineCodeine,
    BrompheniramineCodeinePhenylephrine, BrompheniramineCodeinePseudoephedrine,
    BrompheniramineDextromethorphanPhenylephrine, BrompheniramineDextromethorphanPseudoephedrine,
    BrompheniraminePhenylephrine, BrompheniraminePseudoephedrine, Butabarbital,
    CarbetapentaneChlorpheniramineEphedrinePhenylephrine,
    CarbetapentaneChlorpheniraminePhenylephrine, Carbinoxamine, Carisoprodol,
    ChlophedianolChlorpheniraminePhenylephrine, ChlophedianolDexbrompheniraminePhenylephrine,
    ChlophedianolDexbrompheniraminePseudoephedrine,
    ChlophedianolDexchlorpheniraminePseudoephedrine, Chlorpheniramine, ChlorpheniramineCodeine,
    ChlorpheniramineCodeinePhenylephrine, ChlorpheniramineCodeinePseudoephedrine,
    ChlorpheniramineDextromethorphan, ChlorpheniramineDextromethorphanPhenylephrine,
    ChlorpheniramineDextromethorphanPseudoephedrine, ChlorpheniramineDihydrocodeinePhenylephrine,
    ChlorpheniramineHydrocodone, ChlorpheniramineHydrocodonePseudoephedrine,
    ChlorpheniramineIbuprofenPseudoephedrine, ChlorpheniraminePhenylephrine,
    ChlorpheniraminePhenylephrinePhenyltoloxamine, ChlorpheniraminePhenylephrinePyrilamine,
    ChlorpheniraminePseudoephedrine, Chlorpropamide, Chlorzoxazone, Clemastine, Clomipramine,
    CodeineDexbrompheniraminePseudoephedrine, CodeineDexchlorpheniraminePhenylephrine,
    CodeinePhenylephrinePromethazine, CodeinePromethazine, CodeinePseudoephedrineTriprolidine,
    ConjugatedEstrogens, ConjugatedEstrogensMedroxyprogesterone, CyclobenzaprineHydrochloride,
    DesiccatedThyroid, Dexbrompheniramine, DexbrompheniramineDextromethorphanPhenylephrine,
    DexbrompheniramineDextromethorphanPseudoephedrine,
    DexbrompheniramineMaleatePseudoephedrineHydrochloride, DexbrompheniraminePhenylephrine,
    DexbrompheniraminePseudoephedrine, Dexchlorpheniramine,
    DexchlorpheniramineDextromethorphanPseudoephedrine, DexchlorpheniraminePhenylephrine,
    DexchlorpheniraminePseudoephedrine, DextromethorphanDiphenhydraminePhenylephrine,
    DextromethorphanDoxylamine, DextromethorphanDoxylaminePseudoephedrine,
    DextromethorphanPromethazine, DienogestEstradiolMultiphasic, DiphenhydramineHydrochloride,
    DiphenhydramineIbuprofen, DiphenhydramineMagnesiumSalicylate, DiphenhydramineNaproxen,
    DiphenhydraminePhenylephrine, DiphenhydraminePseudoephedrine, Dipyridamole,
    DischargedToHealthCareFacilityForHospiceCare, DischargedToHomeForHospiceCare, Disopyramide,
    Doxylamine, DoxylaminePhenylephrine, DoxylaminePseudoephedrine, DoxylaminePyridoxine,
    DrospirenoneEstradiol, EncounterInpatient, ErgoloidMesylates, EsterifiedEstrogens,
    EsterifiedEstrogensMethyltestosterone, Estradiol, EstradiolLevonorgestrel,
    EstradiolNorethindrone, EstradiolNorgestimateBiphasic, Estropipate, Ethnicity,
    FaceToFaceInteraction, Glyburide, GlyburideMetformin, Guanfacine,
    HighRiskMedicationsForTheElderly, HighRiskMedicationsWithDaysSupplyCriteria,
    HomeHealthcareServices, HospiceCareAmbulatory, HydrochlorothiazideMethyldopa, Hydroxyzine,
    Imipramine, Indomethacin, Isoxsuprine, KetorolacTromethamine, Megestrol, Meperidine,
    MeperidinePromethazine, Mephobarbital, Meprobamate, Metaxalone, Methocarbamol, Methyldopa,
    NaloxonePentazocine, Nifedipine, NonbenzodiazepineHypnotics, OfficeVisit, OncAdministrativeSex,
    OphthalmologicOutpatientVisit, Orphenadrine, Payer, Pentazocine, Pentobarbital, Phenobarbital,
    PhenylephrinePromethazine, PhenylephrineTriprolidine,
    PreventiveCareServicesEstablishedOfficeVisit18AndUp,
    PreventiveCareServicesInitialOfficeVisit18AndUp, PromethazineHydrochloride,
    PseudoephedrineTriprolidine, Race, Secobarbital, Thioridazine, Ticlopidine, Trihexyphenidyl,
    Trimethobenzamide, Trimipramine, Triprolidine)


class ClinicalQualityMeasure156v6(ClinicalQualityMeasure):
    """
    Use of High-Risk Medications in the Elderly

    Description: Percentage of patients 65 years of age and older who were ordered high-risk
    medications. Two rates are reported.
    a. Percentage of patients who were ordered at least one high-risk medication.
    b. Percentage of patients who were ordered at least two of the same high-risk medications.

    Definition: A high-risk medication is identified by either of the following:
         a. A prescription for medications classified as high risk at any dose and for any duration
         b. Prescriptions for medications classified as high risk at any dose with greater than a
    90 day supply

    Rationale: Seniors receiving inappropriate medications are more likely to report poorer health
    status at follow-up, compared to seniors who receive appropriate medications (Fu, Liu, and
    Christensen 2004. A study of the prevalence of potentially inappropriate medication use in
    older adults found that 40 percent of individuals 65 and older filled at least one prescription
    for a potentially inappropriate medication and 13 percent filled two or more (Fick et al.
    2008). While some adverse drug events are not preventable, studies estimate that between 30 and
    80 percent of adverse drug events in the elderly are preventable (MacKinnon and Hepler 2003).

    Reducing the number of inappropriate prescriptions can lead to improved patient safety and
    significant cost savings.  Conservative estimates of extra costs due to potentially
    inappropriate medications in the elderly average $7.2 billion a year (Fu et al. 2007).
    Medication use by older adults will likely increase further as the U.S. population ages, new
    drugs are developed, and new therapeutic and preventive uses for medications are discovered
    (Rothberg et al. 2008). The annual direct costs of preventable adverse drug events (ADEs) in
    the Medicare population have been estimated to exceed $800 million (IOM, 2007). By the year
    2030, nearly one in five U.S. residents is expected to be aged 65 years or older; this age
    group is projected to more than double in number from 38.7 million in 2008 to more than 88.5
    million in 2050.  Likewise, the population aged 85 years or older is expected to increase
    almost four-fold, from 5.4 million to 19 million between 2008 and 2050.  As the elderly
    population continues to grow, the number of older adults who present with multiple medical
    conditions for which several medications are prescribed continues to increase, resulting in
    polypharmacy (Gray and Gardner 2009).

    Guidance: The intent of Numerator 1 is to assess if the patient has been prescribed at least
    one high-risk medication.

    The intent of Numerator 2 is to assess if the patient has either:
         - Been prescribed at least two of the same high-risk medications.
         - Received two or more prescriptions, where the sum of days supply exceeds 90 days, for
    medications in the same medication class.

    The intent of the measure is to assess if the reporting provider ordered the high-risk
    medication(s). If the patient had a high-risk medication previously prescribed by another
    provider, they would not be counted towards the numerator unless the reporting provider also
    ordered a high-risk medication for them.

    CUMULATIVE MEDICATION DURATION is an individual's total number of medication days over a
    specific period; the period counts multiple prescriptions with gaps in between, but does not
    count the gaps during which a medication was not dispensed.

    To determine the cumulative medication duration, determine first the number of the Medication
    Days for each prescription in the period: the number of doses divided by the dose frequency per
    day. Then add the Medication Days for each prescription without counting any days between the
    prescriptions.

    For example, there is an original prescription for 30 days with 2 refills for thirty days each.
    After a gap of 3 months, the medication was prescribed again for 60 days with 1 refill for 60
    days. The cumulative medication duration is (30 x 3) + (60 x 2) = 210 days over the 10 month
    period.

    More information: https://ecqi.healthit.gov/ecqm/measures/cms156v6
    """

    responds_to_event_types = [
        events.HEALTH_MAINTENANCE,
    ]

    authors = [
        'National Committee for Quality Assurance',
    ]

    references = [
        'Campanelli, Christine M. 2012. "American Geriatrics Society Updated Beers Criteria for Potentially Inappropriate Medication Use in Older Adults: the American Geriatrics Society 2012 Beers Criteria Update Expert Panel." Journal of the American Geriatrics Society 60(4): 616.',
        'American Geriatrics Society 2015 Beers Criteria Update Expert Panel. 2015. "American Geriatrics Society 2015 Updated Beers Criteria for Potentially Inappropriate Medication Use in Older Adults." Journal of the American Geriatrics Society. 63(11): 2227-2246.',
        'Zhan, C, et al. Potentially inappropriate medication use in the community-dwelling elderly. JAMA 2001; 286(22):2823-2868.',
        'Beers, M.H. Explicit criteria for determining potentially inappropriate medication use by the elderly. Arch Intern Med 1997; 157:1531-1536.',
        'Fick, DM, et al. Updating the Beers criteria for potentially inappropriate medication use in older adults. Arch Intern Med 2003; 163:2716-2724.',
        'Fu, A.Z., J.Z. Jiang, J.H. Reeves, J.E. Funcham, G.G. Liu, M. Perri. 2007. "Potentially Inappropriate Medication Use and Healthcare Expenditures in the US Community-Dwelling Elderly." Medical Care 45: 472-6.',
        'Gray, C.L, and C. Gardner. 2009. "Adverse Drug Events in the Elderly: An Ongoing Problem." J Manag Care Pharm 15(7):568-71.',
        'Fick, D.M., L.C. Mion, M.H. Beers, J.L. Waller. 2008. "Health Outcomes Associated with Potentially Inappropriate Medication Use in Older Adults." Research in Nursing & Health. 31(1): 42-51.',
        'AHRQ, "Use of High-Risk Medications in the Elderly: Percentage of Medicare Members 65 years of Age and Older who Received at least two different High-Risk Medications." National Quality Measures Clearinghouse. http://www.qualitymeasures.ahrq.gov/popups/printView.aspx?id=24003 (Accessed Web page: December 14, 2015).',
        'Institute of Medicine (IOM). 2007. Preventing Medication Errors/Committee on Identifying and Preventing Medication Errors. Ed. Aspden P., J.A. Wolcott, J.L. Bootman, L.R. Cronenwatt LR. Quality Chasm Series. Washington, DC: National Academy Press.',
        'MacKinnon, N.J. and C.D. Hepler. 2003. "Indicators of Preventable Drug-related Morbidity in Older Adults: Use Within a Managed Care Organization." J Managed Care Pharm 9:134-41.',
        'Kaufman MB, et al. Effect of Prescriber Education on the Use of Medications Contraindicated in Older Adults in a Managed Medicare Population. J Manag Care Pharm 2005 April/May; 11(3):211-219.',
        'Rothberg, M.B., P.S. Perkow, F. Liu, B. Korc-Grodzicki, M.J. Brennan, S. Bellantonio, M. Heelon, P.K. Lindenauer. 2008. "Potentially Inappropriate Medication Use in Hospitalized Elders." J Hosp Med 3:91-102.',
    ]

    funding_source = ''

    def in_initial_population(self):
        """
        Initial population: Patients 65 years and older who had a visit during the measurement
        period
        """
        age = self.patient.age_at(self.timeframe.end)
        if age < 65:
            return False

        # FIX: check for a visit within the period when we have note data loaded

        return True

    def in_denominator(self):
        """
        Denominator: Equals Initial Population

        Exclusions: Exclude patients who were in hospice care during the measurement year

        Exceptions: None
        """

        # FIX: detect hospice

        return self.in_initial_population()

    def in_numerator(self, population=1):
        """
        Numerator: Numerator 1: Patients with an order for at least one high-risk medication during
        the measurement period

        Numerator 2: Patients with at least two orders for the same high-risk medication during the
        measurement period

        Exclusions: Not Applicable
        """
        if population == 1:
            if self.high_risk_medications():
                return True

        elif population == 2:
            if len(self.high_risk_medications()) >= 2:
                return True

        else:
            raise Exception(f'Unknown population "{population}"')

    def high_risk_medications(self):
        """ find medication orders matching the high risk classification """
        meds = self.patient.medications.within_one_year(self.timeframe)

        # these are high-risk at any dosage
        high_risk = meds.find(HighRiskMedicationsForTheElderly)

        # FIX: these we need to evaluate dosage to decide if they
        # constitute high-risk medications but the medications
        # endpoint isn't returning enough info yet to do that, so
        # we'll err on the side of caution and assume they're high
        # risk too
        high_risk_dosage = meds.find(HighRiskMedicationsWithDaysSupplyCriteria)

        return high_risk + high_risk_dosage

    def compute_results(self):
        """
        Clinical recommendation: The measure is based on recommendations from the American
        Geriatrics Society Beers Criteria for Potentially Inappropriate Medication Use in Older
        Adults. The criteria were developed through key clinical expert consensus processes by
        Beers in 1997, Zahn in 2001 and an updated process by Fick in 2003, 2012 and 2015. The
        Beers Criteria identifies lists of drugs that are potentially inappropriate for all older
        adults and drugs that are potentially inappropriate in the elderly based on various high-
        risk factors such as dosage, days supply and underlying diseases or conditions. NCQA's
        Medication Management expert panel selected a subset of drugs that should be used with
        caution in the elderly for inclusion in the proposed measure based upon the recommendations
        in the Beers Criteria.

        Certain medications (MacKinnon 2003) are associated with increased risk of harm from drug
        side-effects and drug toxicity and pose a concern for patient safety. There is clinical
        consensus that these drugs pose increased risks in the elderly (Kaufman 2005). Studies link
        prescription drug use by the elderly with adverse drug events that contribute to
        hospitalization, increased length of hospital stay, increased duration of illness, nursing
        home placement and falls and fractures that are further associated with physical,
        functional and social decline in the elderly (AHRQ 2009).
        """
        patient = self.patient

        recommendations = []
        status = 'not_applicable'
        narrative = ''

        high_risk = self.high_risk_medications()
        if high_risk:
            names = [med['text'] for med in high_risk]
            narrative = f'{patient.first_name} is elderly and has been prescribed high-risk medication{"s" if len(names) > 1 else ""}: {", ".join(names)}.  The nursing team should perform a medication reconciliation.'

            status = 'due'
            rec = TaskRecommendation(
                patient, narrative=narrative, title='Task: Medication Reconciliation')
            recommendations.append(rec)

        return {'status': status, 'narrative': narrative, 'recommendations': recommendations}



from typing import List, Optional

from .constants import AlertPlacement, AlertIntent


class Intervention:
    """
    A class specifying guidance toward the next course of action within a Protocol.
    """

    def __init__(self,
                 title: Optional[str] = None,
                 narrative: Optional[str] = None,
                 href: str = ''):

        self.title = title
        self.narrative = narrative
        self.href = href


class BannerAlertIntervention(Intervention):
    """
    An intervention that will appear as a banner alert in the UI, specifying a narrative, where it should appear, and what intent it has
    """

    def __init__(
            self, narrative: str, placement: List[AlertPlacement],
            intent: AlertIntent, href: str = ''):

        if len(narrative) > 90 or len(narrative) < 1:
            raise ValueError(
                'Narrative of Banner Alert Intervention must be between 1 and 90 characters')

        placement_options = [p.value for p in AlertPlacement]
        if not isinstance(placement, list):
            raise TypeError(
                f'Placement of Banner Alert Intervention must be a List of strings. The options are: {placement_options}')
        if len(placement) < 1:
            raise ValueError(
                f'Placement of Banner Alert Intervention must have at least one value. The options are: {placement_options}')
        for p in placement:
            if p not in placement_options:
                raise ValueError(
                    f'"{p}" is not an accepted option for Banner Alert Intervention placement. The options are: {placement_options}')

        intent_options = [i.value for i in AlertIntent]
        if intent not in intent_options:
            raise ValueError(
                f'"{intent}" is not an accepted option for Banner Alert Intervention intent. The options are: {intent_options}')

        super().__init__(narrative=narrative, href=href)
        self.placement = placement
        self.intent = intent



from protocols import events
from protocols.protocol import ClinicalQualityMeasure
from protocols.value_set.v2021 import (
    CareServicesInLongTermResidentialFacility, CupToDiscRatio, Ethnicity, NursingFacilityVisit,
    OfficeVisit, OncAdministrativeSex, OphthalmologicalServices,
    OpticDiscExamForStructuralAbnormalities, OutpatientConsultation, Payer,
    PrimaryOpenAngleGlaucoma, Race)


class ClinicalQualityMeasure143v9(ClinicalQualityMeasure):
    """
    Primary Open-Angle Glaucoma (POAG): Optic Nerve Evaluation

    Description: Percentage of patients aged 18 years and older with a diagnosis of primary open-
    angle glaucoma (POAG) who have an optic nerve head evaluation during one or more office visits
    within 12 months

    Definition: None

    Rationale: Glaucoma is a group of diseases that damage the eye’s optic nerve and can result in
    vision loss and blindness. In 2011, 2.71 million persons in the U.S. had primary open-angle
    glaucoma (POAG) and in 2050, an estimated 7.32 million persons will have POAG (Vajaranant, Wu,
    Torres, & Varma, 2012). Furthermore, a study by Rein, Zhang, & Wirth (2006) estimated that the
    total financial burden of major visual disorders among U.S. residents aged 40 years or older
    was $35.4 billion in 2004: $16.2 billion in direct medical costs, $11.1 billion in other direct
    costs, and $8 billion in productivity losses. Of the direct medical costs, approximately $2.9
    billion was attributable to glaucoma (Rein, Zhang, & Wirth, 2006). It is imperative that
    evidence-based care be delivered to all glaucoma patients.

    According to recent guidelines, optic nerve changes are one of the characteristics which
    reflect progression of glaucoma (the other characteristic is visual field). Examination of the
    optic nerve head (ONH) and retinal nerve fiber layer (RNFL) provides valuable structural
    information about optic nerve damage from glaucoma. Visible structural alterations of the ONH
    or RNFL may precede the onset of visual field defects. Careful study of the optic disc neural
    rim for small hemorrhages is important because these hemorrhages sometimes signal focal disc
    damage and visual field loss, and they may signify ongoing optic nerve damage in patients with
    glaucoma (American Academy of Ophthalmology, 2015). Despite evidence emphasizing the value of
    an optic nerve evaluation, there is a gap in documentation patterns of the optic nerve for both
    initial and follow-up care.

    This measure is intended to promote examination and documentation of the structure and function
    of the optic nerve, and to monitor and detect disease progression among patients diagnosed with
    POAG.

    Guidance: Optic nerve head evaluation includes examination of the cup to disc ratio and
    identification of optic disc or retinal nerve abnormalities. Both of these components of the
    optic nerve head evaluation are examined using ophthalmoscopy.

    The measure, as written, does not specifically require documentation of laterality. Coding
    limitations in particular clinical terminologies do not currently allow for that level of
    specificity (ICD-10-CM includes laterality, but SNOMED-CT does not uniformly include this
    distinction). Therefore, at this time, it is not a requirement of this measure to indicate
    laterality of the diagnoses, findings or procedures. Available coding to capture the data
    elements specified in this measure has been provided. It is assumed that the eligible
    professional or eligible clinician will record laterality in the patient medical record, as
    quality care and clinical documentation should include laterality.

    This eCQM is a patient-based measure.

    This version of the eCQM uses QDM version 5.5.  Please refer to the eCQI resource center
    (https://ecqi.healthit.gov/qdm) for more information on the QDM.

    More information: https://ecqi.healthit.gov/ecqm/measures/cms143v9
    """

    title = 'Primary Open-Angle Glaucoma (POAG): Optic Nerve Evaluation'

    identifiers = ['CMS143v9']

    types = ['CQM']

    responds_to_event_types = [
        events.HEALTH_MAINTENANCE,
    ]

    authors = [
        'American Medical Association (AMA)',
        'PCPI(R) Foundation (PCPI[R])',
    ]

    references = [
        'American Academy of Ophthalmology (2015). Primary open-angle glaucoma Preferred Practice Pattern. San Francisco, CA: American Academy of Ophthalmology.',
        'Rein, D. B., Zhang, P., & Wirth, K. (2006). The economic burden of major adult visual disorders in the United States. Archives of Ophthalmology, 124(12), 1754-1760. doi:10.1001/archopht.124.12.1754',
        'Vajaranant, T. S., Wu, S., Torres, M., & Varma, R. (2012). The changing face of primary open-angle glaucoma in the United States: Demographic and geographic changes from 2011 to 2050. American Journal of Ophthalmology, 154(2). doi:10.1016/j.ajo.2012.02.024',
    ]

    funding_source = ''

    def in_initial_population(self):
        """
        Initial population: All patients aged 18 years and older with a diagnosis of primary open-
        angle glaucoma
        """
        pass

    def in_denominator(self):
        """
        Denominator: Equals Initial Population

        Exclusions: None

        Exceptions: Documentation of medical reason(s) for not performing an optic nerve head
        evaluation
        """
        return self.in_initial_population()

    def in_numerator(self):
        """
        Numerator: Patients who have an optic nerve head evaluation during one or more office
        visits within 12 months

        Exclusions: Not Applicable
        """
        pass


    def compute_results(self):
        """
        Clinical recommendation: Ophthalmic Evaluation
        The ophthalmic evaluation specifically focuses on the following elements in the
        comprehensive adult medical eye evaluation:

        Visual acuity measurement
        Pupil examination
        Anterior segment examination
        IOP measurement
        Gonioscopy
        Optic nerve head (ONH) and retinal nerve fiber layer (RNFL) examination
        Fundus examination
        (American Academy of Ophthalmology, 2015)

        The optic nerve should be carefully examined for the signs of glaucoma damage, and its
        appearance should be serially documented (I+, moderate quality, strong recommendation)
        (American Academy of Ophthalmology, 2015).
        """
        pass



# Ensure value sets can all be imported


import importlib
from pathlib import Path
from unittest import TestCase
from canvas_workflow_kit import value_set


class SuperValueSetTest(TestCase):

    def test_valuesets(self):
        """
        Test value sets to ensure they compile and can be imported.
        """

        main_path = Path(value_set.__path__[0])

        errors = []

        def test_directory(dir_path):
            for item in dir_path.iterdir():
                if "__pycache__" in str(item):
                    continue

                if item.is_dir():
                    test_directory(item)
                    continue

                if item.suffix == '.py':
                    try:
                        module = importlib.util.spec_from_file_location("canvas_workflow_kit.value_set.v202X", item)
                        module.loader.load_module()
                    except SyntaxError as e:
                        errors.append(f"Syntax error in valueset file: {item} " + str(e))

        test_directory(main_path)
        if errors:
            self.fail('\n'.join(errors))



from .base import SDKBaseTest
from canvas_workflow_kit.action import Coding, Action


class CodingTest(SDKBaseTest):

    def test___init__(self):
        # placeholder test
        tested = Coding()
        self.assertIsInstance(tested, Coding)


class ActionTest(SDKBaseTest):

    def test___init__(self):
        tested = Action('Title', 'Type', 'This is some content')
        self.assertIsInstance(tested, Action)
        self.assertEqual('Title', tested.title)
        self.assertEqual('Type', tested.type)
        self.assertEqual('This is some content', tested.content)

    def test_to_json(self):
        # placeholder test
        tested = Action('Title', 'Type', 'This is some content')
        self.assertIsInstance(tested, Action)
        result = tested.to_json()
        self.assertIsNone(result)



from protocols import events
from protocols.protocol import ClinicalQualityMeasure
from protocols.value_set.v2021 import (
    AdhdMedications, BehavioralHealthFollowUpVisit, EncounterInpatient, Ethnicity,
    HomeHealthcareServices, HospiceCareAmbulatory, HospitalObservationCareInitial, Narcolepsy,
    OfficeVisit, OncAdministrativeSex, OutpatientConsultation, Payer,
    PreventiveCareEstablishedOfficeVisit0To17, PreventiveCareServicesGroupCounseling,
    PreventiveCareServicesIndividualCounseling, PreventiveCareServicesInitialOfficeVisit0To17,
    PsychVisitDiagnosticEvaluation, PsychVisitPsychotherapy,
    PsychotherapyAndPharmacologicManagement, Race, TelehealthServices, TelephoneManagement)


class ClinicalQualityMeasure136v10(ClinicalQualityMeasure):
    """
    Follow-Up Care for Children Prescribed ADHD Medication (ADD)

    Description: Percentage of children 6-12 years of age and newly dispensed a medication for
    attention-deficit/hyperactivity disorder (ADHD) who had appropriate follow-up care.  Two rates
    are reported.
    a. Percentage of children who had one follow-up visit with a practitioner with prescribing
    authority during the 30-Day Initiation Phase.
    b. Percentage of children who remained on ADHD medication for at least 210 days and who, in
    addition to the visit in the Initiation Phase, had at least two additional follow-up visits
    with a practitioner within 270 days (9 months) after the Initiation Phase ended.

    Definition: Intake Period: The five-month period starting 90 days prior to the start of the
    measurement period and ending 60 days after the start of the measurement period.

    Index Prescription Start Date (IPSD): The earliest prescription dispensing date for an ADHD
    medication where the date is in the Intake Period and an ADHD medication was not dispensed
    during the 120 days prior.

    Initiation Phase: The 30 days following the IPSD.

    Continuation and Maintenance Phase: The 31-300 days following the IPSD.

    Rationale: Attention-deficit/hyperactivity disorder (ADHD) is the most common neurobehavioral
    disorder of childhood and can profoundly affect the academic achievement, well-being, and
    social interactions of children (American Academy of Pediatrics, 2011). The American
    Psychiatric Association states in the Diagnostic and Statistical Manual of Mental Disorders
    that five percent of children have ADHD (American Psychiatric Association, 2013). However,
    other studies in the US have estimated higher rates in community samples.  For example, a study
    examining data from the National Survey of Children's Health estimated that approximately 9.4%
    of children 2-17 years of age (6.1 million) had ever been diagnosed with ADHD, according to
    parent report in 2016 (Danielson et al., 2016). Among those children, 6 out of 10 (62%) were
    taking medication for their ADHD and represent 1 out of 20 of all U.S. children. Just under
    half (47%) received any behavioral treatment for their ADHD in the past year (Danielson et al.,
    2016).

    There are many symptoms associated with ADHD. Children with ADHD may experience significant
    functional problems, such as school difficulties, academic underachievement, troublesome
    relationships with family members and peers and behavioral problems (American Academy of
    Pediatrics, 2000). For instance, recent studies have found that parents whose children have a
    history of ADHD report significantly more peer problems and a higher rate of non-fatal injuries
    compared to parents whose children do not have a history of ADHD (Strine et al., 2006; Xiang et
    al., 2005). Additional studies suggest that there is an increased risk for drug use disorders
    in adolescents with untreated ADHD (National Institute on Drug Abuse, 2010). One of the
    national objectives of the Department of Health and Human Services Healthy People 2020
    initiative is to increase the proportion of children with mental health problems who receive
    treatment.

    Medication treatment has been found to be effective for managing ADHD, but requires careful
    monitoring by physicians. Studies have shown that psychostimulants are highly effective for
    75-90% of children with ADHD by reducing symptoms of hyperactivity, impulsivity and
    inattention; improving classroom performance and behavior; and promoting increased interaction
    with teachers, parents and peers (U.S. Department of Health and Human Services, 1999). Some
    reported adverse effects of stimulant ADHD medications include appetite loss, abdominal pain,
    headaches, sleep disturbance, decreasing growth velocity, and less commonly, hallucinations and
    other psychotic symptoms. Additionally, treatments for children with ADHD are frequently not
    sustained despite the fact that they are at greater risk of significant problems if they
    discontinue treatment (Wolraich et al., 2011). Effective management mitigates the risk of
    discontinuing treatment.

    The intent of this measure is to ensure timely and continuous follow-up visits for children who
    are newly prescribed ADHD medication. The goal is to encourage monitoring of children for
    medication effectiveness, occurrence of side effects and adherence.

    Guidance: CUMULATIVE MEDICATION DURATION is an individual's total number of medication days
    over a specific period; the period counts multiple prescriptions with gaps in between, but does
    not count the gaps during which a medication was not dispensed.

    To determine the cumulative medication duration, determine first the number of the medication
    Days for each prescription in the period: the number of doses divided by the dose frequency per
    day. Then add the Medication Days for each prescription without counting any days between the
    prescriptions.

    For example, there is an original prescription for 30 days with 2 refills for thirty days each.
    After a gap of 3 months, the medication was prescribed again for 60 days with 1 refill for 60
    days. The cumulative medication duration is (30 x 3) + (60 x 2) = 210 days over the 10 month
    period.

    This eCQM is a patient-based measure.

    This version of the eCQM uses QDM version 5.5. Please refer to the eCQI resource center
    (https://ecqi.healthit.gov/qdm) for more information on the QDM.

    More information: https://ecqi.healthit.gov/ecqm/measures/cms136v10
    """

    title = 'Follow-Up Care for Children Prescribed ADHD Medication (ADD)'

    identifiers = ['CMS136v10']

    types = ['CQM']

    responds_to_event_types = [
        events.HEALTH_MAINTENANCE,
    ]

    authors = [
        'National Committee for Quality Assurance',
    ]

    references = [
        'American Academy of Pediatrics. (2011, November). ADHD: Clinical practice guideline for the diagnosis, evaluation, and treatment of attention-deficit/hyperactivity disorder in children and adolescents. Pediatrics, 128(5), 1007-1022.',
        'American Academy of Pediatrics. (2000). Clinical practice guideline: Diagnosis and evaluation of the child with attention-deficit/hyperactivity disorder. Pediatrics, 105(5), 1158-1170.',
        'American Psychiatric Association. (2013). Diagnostic and statistical manual of mental disorders, 5th edition (DSM-5). Washington, DC: Author.',
        'Danielson, M. L., Bitsko, R. H., Ghandour, R. M., et al. (2016). Prevalence of parent-reported ADHD diagnosis and associated treatment among U.S. children and adolescents. Journal of Clinical Child & Adolescent Psychology, 47(2), 199-212.',
        'Jensen, P., Hinshaw, S. P., Swanson, J. M., et al. (2001). Findings from the NIMH multimodal treatment study of ADHD (MTA): Implications and applications for primary care providers. Journal of Developmental and Behavioral Pediatrics, 22(1), 60-73.',
        'National Institute on Drug Abuse. (2010, September). Common Comorbidities with Substance Use Disorders. Retrieved from http://www.drugabuse.gov/publications/research-reports/comorbidity-addiction-other-mental-illnesses/how-common-are-comorbid-drug-use-other-mental-diso',
        'Pliszka, S., & AACAP Work Group on Quality Issues. (2007). Practice parameter for the assessment and treatment of children and adolescents with attention-deficit/hyperactivity disorder. Journal of the American Academy of Child and Adolescent Psychiatry, 46(7), 894-921.',
        'Robb, J. A., Sibley, M. H., Pelham, W. E., Jr., et al. (2011, September). The estimated annual cost of ADHD to the U.S. education system. School Mental Health, 3(3), 169-177. Retrieved from http://link.springer.com/article/10.1007/s12310-011-9057-6#',
        'Strine, T. W., Lesesne, C. A., Okoro, C. A., et al. (2006). Emotional and behavioral difficulties and impairments in everyday functioning among children with a history of attention-deficit/hyperactivity disorder. Preventing Chronic Disease, 3(2), A52.',
        'Swensen, A. R., Birnbaum, H. G., Secnik, K., et al. (2003). Attention-deficit/hyperactivity disorder: Increased costs for patients and their families. Journal of the American Academy of Child Adolescent Psychiatry, 42(12), 1415-1423.',
        'U.S. Department of Health and Human Services. (1999). Mental health: A report of the surgeon general. Retrieved from http://profiles.nlm.nih.gov/ps/retrieve/ResourceMetadata/NNBBHS',
        'Wolraich, M., Brown, L., Brown, R. T., et al. (2011). ADHD: clinical practice guideline for the diagnosis, evaluation, and treatment of attention-deficit/hyperactivity disorder in children and adolescents. Pediatrics, 128(5), 1007-1022.',
        'Xiang, H., Stallones, L., Chen, G., et al. (2005). Nonfatal injuries among U.S. children with disabling conditions: Opportunity for improvement. American Journal of Public Health, 95(11), 1970-1975.',
    ]

    funding_source = ''

    def in_initial_population(self):
        """
        Initial population: Initial Population 1: Children 6-12 years of age who were dispensed an
        ADHD medication during the Intake Period and who had a visit during the measurement period.

        Initial Population 2: Children 6-12 years of age who were dispensed an ADHD medication
        during the Intake Period and who remained on the medication for at least 210 days out of
        the 300 days following the IPSD, and who had a visit during the measurement period.
        """
        pass

    def in_denominator(self):
        """
        Denominator: Equals Initial Population

        Exclusions: Denominator Exclusion 1: Exclude patients diagnosed with narcolepsy at any
        point in their history or during the measurement period.

        Exclude patients who had an acute inpatient stay with a principal diagnosis of mental
        health or substance abuse during the 30 days after the IPSD.

        Exclude patients who were actively on an ADHD medication in the 120 days prior to the Index
        Prescription Start Date.

        Exclude patients whose hospice care overlaps the measurement period.

        Denominator Exclusion 2: Exclude patients diagnosed with narcolepsy at any point in their
        history or during the measurement period.

        Exclude patients who had an acute inpatient stay with a principal diagnosis of mental
        health or substance abuse during the 300 days after the IPSD.

        Exclude patients who were actively on an ADHD medication in the 120 days prior to the Index
        Prescription Start Date.

        Exclude patients whose hospice care overlaps the measurement period.

        Exceptions: None
        """
        return self.in_initial_population()

    def in_numerator(self):
        """
        Numerator: Numerator 1: Patients who had at least one face-to-face visit with a
        practitioner with prescribing authority within 30 days after the IPSD.

        Numerator 2: Patients who had at least one face-to-face visit with a practitioner with
        prescribing authority during the Initiation Phase, and at least two follow-up visits during
        the Continuation and Maintenance Phase. One of the two visits during the Continuation and
        Maintenance Phase may be a telephone visit with a practitioner.

        Exclusions: Not Applicable
        """
        pass


    def compute_results(self):
        """
        Clinical recommendation: American Academy of Child and Adolescent Psychiatry (AACAP)
        Practice Parameter for the Assessment and Treatment of Children and Adolescents with ADHD

        - Overall Guideline
        The key to effective long-term management of the patient with ADHD is continuity of care
        with a clinician experienced in the treatment of ADHD. The frequency and duration of
        follow-up sessions should be individualized for each family and patient, depending on the
        severity of ADHD symptoms; the degree of comorbidity of other psychiatric illness; the
        response to treatment; and the degree of impairment in home, school, work, or peer-related
        activities. The clinician should establish an effective mechanism for receiving feedback
        from the family and other important informants in the patient's environment to be sure
        symptoms are well controlled and side effects are minimal. Although this parameter does not
        seek to set a formula for the method of follow-up, significant contact with the clinician
        should typically occur two to four times per year in cases of uncomplicated ADHD and up to
        weekly sessions at times of severe dysfunction or complications of treatment.

        - Recommendation 6: A Well-Thought-Out and Comprehensive Treatment Plan Should Be Developed
        for the Patient With ADHD. The treatment plan should be reviewed regularly and modified if
        the patient's symptoms do not respond. Minimal Standard [MS]

        - Recommendation 9. During a Psychopharmacological Intervention for ADHD, the Patient
        Should Be Monitored for Treatment-Emergent Side Effects. Minimal Standard [MS]

        - Recommendation 12. Patients Should Be Assessed Periodically to Determine Whether There Is
        Continued Need for Treatment or If Symptoms Have Remitted. Treatment of ADHD Should
        Continue as Long as Symptoms Remain Present and Cause Impairment. Minimal Standard [MS]

        American Academy of Pediatrics Clinical Practice Guideline for the Diagnosis, Evaluation
        and Treatment of ADHD in Children and Adolescents

        - Action Statement 4: The primary care clinician should recognize ADHD as a chronic
        condition and, therefore, consider children and adolescents with ADHD as children and youth
        with special health care needs. Management of children and youth with special health care
        needs should follow the principles of the chronic care model and the medical home. Grade B:
        Strong Recommendation

        Additionally, in the supplemental information provided along with the 2011 American Academy
        of Pediatrics Guideline, the following recommendations are made:
        - “A face-to-face follow-up visit is recommended by the fourth week of medication, during
        which clinicians review the responses to the varying doses and monitor adverse effects,
        pulse, blood pressure, and weight.”
        - “Subsequent visits will depend on the response but should occur at least 2 times per
        year, until it is clear that target goals are progressing and stable, and then periodically
        as determined by the family and the clinician.”
        """
        pass



from functools import singledispatch
from .string import snakecase


class AttrDict(dict):
    """A dictionary with attribute-style access. It maps attribute access to
    the real dictionary.  """

    def __init__(self, init={}):
        dict.__init__(self, init)

    def __getstate__(self):
        return self.__dict__.items()

    def __setstate__(self, items):
        for key, val in items:
            self.__dict__[key] = val

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, dict.__repr__(self))

    def __setitem__(self, key, value):
        return super(AttrDict, self).__setitem__(key, value)

    def __getitem__(self, name):
        return super(AttrDict, self).__getitem__(name)

    def __delitem__(self, name):
        return super(AttrDict, self).__delitem__(name)

    def __dir__(self):
        """Replace dict autocomplete choices with dict keys"""
        return self.keys()

    __getattr__ = __getitem__
    __setattr__ = __setitem__


@singledispatch
def to_attr_dict(arg):
    """
    Recursively converts an object or list of objects to an attribute dict
    with snake case
    """
    return arg


@to_attr_dict.register
def _(arg: list):
    return [to_attr_dict(i) for i in arg]


@to_attr_dict.register
def _(arg: dict):
    return AttrDict({snakecase(k): to_attr_dict(v) for k, v in arg.items()})



# type: ignore
from typing import Any, Dict, List, Optional

import arrow

from cached_property import cached_property

from canvas_workflow_kit import events
from canvas_workflow_kit.protocol import (
    STATUS_DUE,
    STATUS_SATISFIED,
    ClinicalQualityMeasure,
    ExternallyAwareClinicalQualityMeasure,
    ProtocolResult
)
from canvas_workflow_kit.recommendation import InstructionRecommendation, LabRecommendation
# flake8: noqa
from canvas_workflow_kit.value_set.v2018 import (
    Diabetes,
    DietaryRecommendations,
    Hba1CLaboratoryTest
)
from canvas_workflow_kit.builtin_cqms.diabetes_quality_measure import DiabetesQualityMeasure


class ClinicalQualityMeasure122v6(ExternallyAwareClinicalQualityMeasure, DiabetesQualityMeasure):
    """
    Diabetes: Hemoglobin A1c (HbA1c) Poor Control (> 9%)

    Description: Percentage of patients 18-75 years of age with diabetes who had hemoglobin A1c >
    9.0% during the measurement period

    Definition: None

    Rationale: As the seventh leading cause of death in the U.S., diabetes kills approximately
    75,000 people a year (CDC FastStats 2015). Diabetes is a group of diseases marked by high blood
    glucose levels, resulting from the body's inability to produce or use insulin (CDC Statistics
    2014, ADA Basics 2013). People with diabetes are at increased risk of serious health
    complications including vision loss, heart disease, stroke, kidney failure, amputation of toes,
    feet or legs, and premature death. (CDC Fact Sheet 2014).

    In 2012, diabetes cost the U.S. an estimated $245 billion: $176 billion in direct medical costs
    and $69 billion in reduced productivity. This is a 41 percent increase from the estimated $174
    billion spent on diabetes in 2007 (ADA Economic 2013).

    Reducing A1c blood level results by 1 percentage point (eg, from 8.0 percent to 7.0 percent)
    helps reduce the risk of microvascular complications (eye, kidney and nerve diseases) by as
    much as 40 percent (CDC Estimates 2011).

    Guidance: Patient is numerator compliant if most recent HbA1c level >9%, the most recent HbA1c
    result is missing, or if there are no HbA1c tests performed and results documented during the
    measurement period. If the HbA1c test result is in the medical record, the test can be used to
    determine numerator compliance.

    Only patients with a diagnosis of Type 1 or Type 2 diabetes should be included in the
    denominator of this measure; patients with a diagnosis of secondary diabetes due to another
    condition should not be included.

    More information: https://ecqi.healthit.gov/sites/default/files/ecqm/measures/CMS122v6.html
    """

    class Meta:
        title = 'Diabetes: Hemoglobin HbA1c Poor Control (> 9%)'

        version = '2019-02-12v1'

        description = (
            'Patients 18-75 years of age with diabetes who have either a hemoglobin A1c > 9.0% '
            'or no hemoglobin A1c within the last year.')
        information = 'https://ecqi.healthit.gov/sites/default/files/ecqm/measures/CMS122v6.html'

        identifiers = ['CMS122v6']

        types = ['CQM']

        responds_to_event_types = [
            events.HEALTH_MAINTENANCE,
        ]

        authors = [
            'National Committee for Quality Assurance',
        ]
        references = [
            'American Diabetes Association. Glycemic targets. Sec. 6. In Standards of Medical Care in Diabetes-2017. Diabetes Care 2017;40(Suppl. 1):S48-S56',
            'American Diabetes Association. 2013. Diabetes Basics. www.diabetes.org/diabetes-basics/?loc=GlobalNavDB',
            'American Diabetes Association (ADA). April 2013. Economic Costs of Diabetes in the U.S. in 2012. Diabetes Care. Vol. 36 no. 41033-46. http://care.diabetesjournals.org/content/36/4/1033.full',
            'Centers for Disease Control and Prevention (CDC). 2014. National Diabetes Statistics Report. http://www.cdc.gov/diabetes/pdfs/data/2014-report-estimates-of-diabetes-and-its-burden-in-the-united-states.pdf',
            'Centers for Disease Control and Prevention (CDC). 2015. FastStats: Deaths and Mortality. www.cdc.gov/nchs/fastats/deaths.htm.',
            'Centers for Disease Control and Prevention. 2011. National diabetes fact sheet: national estimates and general information on diabetes and prediabetes in the United States, 2011. Atlanta, GA: U.S. Department of Health and Human Services, Centers for Disease Control and Prevention. www.cdc.gov/diabetes/pubs/pdf/ndfs_2011.pdf',
            'Centers for Disease Control and Prevention. 2014. CDC Features. Diabetes Latest. www.cdc.gov/features/diabetesfactsheet/.', ]

        compute_on_change_types = [
            ClinicalQualityMeasure.CHANGE_PROTOCOL_OVERRIDE,
            ClinicalQualityMeasure.CHANGE_CONDITION,
            ClinicalQualityMeasure.CHANGE_LAB_REPORT,
            ClinicalQualityMeasure.CHANGE_PATIENT,
        ]

    @classmethod
    def enabled(cls) -> bool:
        return True

    MINIMUM_HBA1C = 9.0

    @cached_property
    def last_hba1c_record(self) -> Dict[str, Any]:
        return (self.patient.lab_reports
                .find(Hba1CLaboratoryTest)
                .within(self.timeframe)
                .last())  # yapf: disable

    @property
    def last_hba1c_value(self) -> float:
        # TODO based on solution adopted through https://github.com/canvas-medical/canvas/issues/4799
        if self.last_hba1c_record:
            if isinstance(self.last_hba1c_record['value'], str):
                return self.relative_float(self.last_hba1c_record['value'])
            else:
                return float(self.last_hba1c_record['value'])

    @property
    def last_hba1c_arrow(self) -> Optional[arrow.Arrow]:
        if self.last_hba1c_record:
            return arrow.get(self.last_hba1c_record['originalDate'])

    @property
    def last_hba1c_date(self) -> str:
        if self.last_hba1c_record:
            return self.display_date(arrow.get(self.last_hba1c_record['originalDate']))

    def in_denominator(self) -> bool:
        """
        Denominator: Equals Initial Population

        Exclusions: Exclude patients who were in hospice care during the measurement year

        Exceptions: None
        """
        if not self.in_initial_population():
            return False

        if self.patient.hospice_within(self.timeframe):
            return False

        return True

    def in_numerator(self) -> bool:
        """
        Numerator:
         + patients without test performed in the measurement period
         + patients with a performed test in the period but without result
         + patients whose most recent HbA1c level (performed during the measurement period) is >9.0%

        Exclusions: Not Applicable
        """
        if not self.last_hba1c_record:
            return True

        return self.last_hba1c_value > self.MINIMUM_HBA1C

    def craft_unsatisfied_result(self):
        result = ProtocolResult()

        result.due_in = (self.last_hba1c_arrow.shift(days=self.timeframe.duration) - self.now).days
        result.status = STATUS_SATISFIED

        result.add_narrative(
            f"{self.patient.first_name}'s last HbA1c done {self.last_hba1c_date} was "
            f'{self.last_hba1c_value:.1f}%.')

        return result

    def craft_satisfied_result(self):
        """
        Clinical recommendation: American Diabetes Association (2017):

        - A reasonable A1C goal for many nonpregnant adults is <7%. (Level of evidence: A)

        - Providers might reasonably suggest more stringent A1C goals (such as <6.5%) for selected
        individual patients if this can be achieved without significant hypoglycemia or other
        adverse effects of treatment. Appropriate patients might include those with short duration
        of diabetes, type 2 diabetes treated with lifestyle or metformin only, long life
        expectancy, or no significant cardiovascular disease (CVD). (Level of evidence: C)

        - Less stringent A1C goals (such as <8%) may be appropriate for patients
        with a history of severe hypoglycemia, limited life expectancy, advanced microvascular or
        macrovascular complications, extensive comorbid
        conditions, or long-standing diabetes in whom the general goal is difficult to attain
        despite diabetes self-management education, appropriate glucose monitoring, and effective
        doses of multiple glucose-lowering agents including insulin. (Level of evidence: B)
        """
        result = ProtocolResult()

        context = {
            'conditions': [[{
                'code': 'Z131',
                'system': 'ICD-10',
                'display': 'Encounter for screening for diabetes mellitus',
            }]]
        }

        first_name = self.patient.first_name

        result.due_in = -1
        result.status = STATUS_DUE

        if self.last_hba1c_value is None:
            result.add_narrative("{0}'s last HbA1c test was over {1}.".format(
                first_name,
                self.now.shift(days=-1 * self.timeframe.duration, months=-1).humanize(
                    other=self.now, granularity='month', only_distance=True)).replace(' ago', ''))

            result.add_recommendation(
                LabRecommendation(
                    key='CMS122v6_RECOMMEND_HBA1C',
                    rank=1,
                    button='Order',
                    patient=self.patient,
                    context=context,
                    lab=Hba1CLaboratoryTest,
                    condition=Diabetes,
                    title='Order HbA1c'))
        else:
            result.add_narrative(f"{first_name}'s last HbA1c done {self.last_hba1c_date} was "
                                 f'{self.last_hba1c_value:.1f}%.')

            title = ('Discuss lifestyle modification and medication adherence. '
                     'Consider diabetes education and medication intensification as appropriate.')

            result.add_recommendation(
                InstructionRecommendation(
                    key='CMS122v6_RECOMMEND_DISCUSS_LIFESTYLE',
                    rank=1,
                    button='Instruct',
                    patient=self.patient,
                    instruction=DietaryRecommendations,
                    title=title))

        return result
